#include "StdAfx.h"
#include "ImageTexture.h"

ImageTexture::ImageTexture()
{
	hIcon = 0;

	useSmallIcon = false;

	memoryDC = 0;
	memoryBitmap = 0;

	memoryHoverDC = 0;
	memoryHoverBitmap = 0;

	memoryPressedDC = 0;
	memoryPressedBitmap = 0;

	memoryAnimDC = 0;
	memoryAnimBitmap = 0; 

	memoryHoverAnimDC = 0;
	memoryHoverAnimBitmap = 0; 

	transparencyNeeded = false;

	UseAlphaMap = false;

	has_AlphaMap = false;
	has_hoverAlphaMap = false;
	has_pressedAlphaMap = false;
	has_animAlphaMap = false;
	has_hoveranimAlphaMap = false;
}

ImageTexture::~ImageTexture()
{
	if(hIcon)
		DestroyIcon(hIcon);

	if(memoryBitmap)
	{
		memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
		DeleteObject(memoryBitmap);
	}
	if(memoryDC)
		DeleteDC(memoryDC);

	if(memoryHoverBitmap)
	{
		memoryHoverBitmap = (HBITMAP) SelectObject(memoryHoverDC, memoryHoverBitmap);
		DeleteObject(memoryHoverBitmap);
	}
	if(memoryHoverDC)
		DeleteDC(memoryHoverDC);

	if(memoryPressedBitmap)
	{
		memoryPressedBitmap = (HBITMAP) SelectObject(memoryPressedDC, memoryPressedBitmap);
		DeleteObject(memoryPressedBitmap);
	}
	if(memoryPressedDC)
		DeleteDC(memoryPressedDC);

	if(memoryAnimBitmap)
	{
		memoryAnimBitmap = (HBITMAP) SelectObject(memoryAnimDC, memoryAnimBitmap);
		DeleteObject(memoryAnimBitmap);
	}
	if(memoryAnimDC)
		DeleteDC(memoryAnimDC);

	if(memoryHoverAnimBitmap)
	{
		memoryHoverAnimBitmap = (HBITMAP) SelectObject(memoryHoverAnimDC, memoryHoverAnimBitmap);
		DeleteObject(memoryHoverAnimBitmap);
	}
	if(memoryHoverAnimDC)
		DeleteDC(memoryHoverAnimDC);
}

void ImageTexture::configure(const string &prefix, const string &subKey)
{
	NameValuePair modeConstants[] =
	{
		{ "stretch", MULTIBLT_STRETCH },
		{ "tile", MULTIBLT_TILE },
		{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tile-vertical", MULTIBLT_TILEVERTICAL },
		{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tilevertical", MULTIBLT_TILEVERTICAL },
		{ 0, 0 }
	};

	if ( IsOS(OS_2KXP) )
		UseAlphaMap = GetxRCBool(prefix, "AlphaMap", GetxRCBool(subKey, "AlphaMap", GetxRCBool("AllLabels", "AlphaMap")));
	else
		UseAlphaMap = false;

	string imageFile = GetxRCString(prefix, "Image", GetxRCString(subKey, "Image", GetxRCString("AllLabels", "Image", "")));
	imageFile = lowerCase(imageFile);

	if ( strstr(imageFile.c_str(), ".bmp") != NULL || strstr(imageFile.c_str(), ".png") != NULL || strstr(imageFile.c_str(), ".extract=") != NULL)
		memoryBitmap = LoadLSImage(imageFile.c_str(), 0);
	else
	{	
		useSmallIcon = GetxRCBool(prefix, "UseSmallIcon", GetxRCBool(subKey, "UseSmallIcon", GetxRCBool("AllLabels", "UseSmallIcon")));

		SHFILEINFO shFileInfo;
		SHGetFileInfo(imageFile.c_str(), NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (useSmallIcon ? SHGFI_SMALLICON : 0) );
		hIcon = shFileInfo.hIcon;

		transparencyNeeded = true;
	}

	if (hIcon == 0)
	{
		if (UseAlphaMap)
		{
			BITMAP bm;
			GetObject(memoryBitmap, sizeof(BITMAP), &bm);
			has_AlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

			if (has_AlphaMap)
				PrepareAlphaChannel(memoryBitmap);
		}

		GetLSBitmapSize(memoryBitmap, &imageWidth, &imageHeight);

		memoryDC = CreateCompatibleDC(0);
		SelectObject(memoryDC, memoryBitmap);

		imageFile = GetxRCString(prefix, "HoverImage", GetxRCString(subKey, "HoverImage", GetxRCString("AllLabels", "HoverImage", "")));
		if (!imageFile.empty())
		{
			memoryHoverBitmap = LoadLSImage(imageFile.c_str(), 0);

			if (UseAlphaMap && memoryHoverBitmap)
			{
				BITMAP bm;
				GetObject(memoryHoverBitmap, sizeof(BITMAP), &bm);
				has_hoverAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

				if (has_hoverAlphaMap)
					PrepareAlphaChannel(memoryHoverBitmap);
			}
		}

		if (memoryHoverBitmap)
		{
			GetLSBitmapSize(memoryHoverBitmap, &hoverimageWidth, &hoverimageHeight);

			memoryHoverDC = CreateCompatibleDC(0);
			SelectObject(memoryHoverDC, memoryHoverBitmap);
		}

		imageFile = GetxRCString(prefix, "PressedImage", GetxRCString(subKey, "PressedImage", GetxRCString("AllLabels", "PressedImage", "")));
		if (!imageFile.empty())
		{
			memoryPressedBitmap = LoadLSImage(imageFile.c_str(), 0);

			if (UseAlphaMap && memoryPressedBitmap)
			{
				BITMAP bm;
				GetObject(memoryPressedBitmap, sizeof(BITMAP), &bm);
				has_pressedAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

				if (has_pressedAlphaMap)
					PrepareAlphaChannel(memoryPressedBitmap);
			}
		}

		if (memoryPressedBitmap)
		{
			GetLSBitmapSize(memoryPressedBitmap, &pressedimageWidth, &pressedimageHeight);

			memoryPressedDC = CreateCompatibleDC(0);
			SelectObject(memoryPressedDC, memoryPressedBitmap);
		}

		saturation = GetxRCInt(prefix , "SaturationIntensity", GetxRCInt(subKey, "SaturationIntensity", GetxRCInt("AllLabels" , "SaturationIntensity", 255, 0, 255), 0, 255), 0, 255);
		hueIntensity = GetxRCInt(prefix , "HueIntensity", GetxRCInt(subKey, "HueIntensity", GetxRCInt("AllLabels" , "HueIntensity", 0, 0, 255), 0, 255), 0, 255);
		hueColor = GetxRCColor(prefix, "HueColor", GetxRCColor(subKey, "HueColor", GetxRCColor("AllLabels", "HueColor", RGB(0, 0, 0))));

		hoversaturation = GetxRCInt(prefix , "HoverSaturationIntensity", GetxRCInt(subKey, "HoverSaturationIntensity", GetxRCInt("AllLabels" , "HoverSaturationIntensity", 255, 0, 255), 0, 255), 0, 255);
		hoverhueIntensity = GetxRCInt(prefix , "HoverHueIntensity", GetxRCInt(subKey, "HoverHueIntensity", GetxRCInt("AllLabels" , "HoverHueIntensity", 0, 0, 255), 0, 255), 0, 255);
		hoverhueColor = GetxRCColor(prefix, "HoverHueColor", GetxRCColor(subKey, "HoverHueColor", GetxRCColor("AllLabels", "HoverHueColor", hueColor)));

		pressedsaturation = GetxRCInt(prefix , "PressedSaturationIntensity", GetxRCInt(subKey, "PressedSaturationIntensity", GetxRCInt("AllLabels" , "PressedSaturationIntensity", 255, 0, 255), 0, 255), 0, 255);
		pressedhueIntensity = GetxRCInt(prefix , "PressedHueIntensity", GetxRCInt(subKey, "PressedHueIntensity", GetxRCInt("AllLabels" , "PressedHueIntensity", 0, 0, 255), 0, 255), 0, 255);
		pressedhueColor = GetxRCColor(prefix, "PressedHueColor", GetxRCColor(subKey, "PressedHueColor", GetxRCColor("AllLabels", "PressedHueColor", hoverhueColor)));

		leftEdge = GetxRCInt(prefix, "ImageLeftEdge", GetxRCInt(subKey, "ImageLeftEdge", GetxRCInt("AllLabels", "ImageLeftEdge", 0, 0), 0), 0);
		topEdge = GetxRCInt(prefix, "ImageTopEdge", GetxRCInt(subKey, "ImageTopEdge", GetxRCInt("AllLabels", "ImageTopEdge", 0, 0), 0), 0);
		rightEdge = GetxRCInt(prefix, "ImageRightEdge", GetxRCInt(subKey, "ImageRightEdge", GetxRCInt("AllLabels", "ImageRightEdge", 0, 0), 0), 0);
		bottomEdge = GetxRCInt(prefix, "ImageBottomEdge", GetxRCInt(subKey, "ImageBottomEdge", GetxRCInt("AllLabels", "ImageBottomEdge", 0, 0), 0), 0);

		mode = GetxRCNamedValue(prefix, "ImageMode", modeConstants, GetxRCNamedValue(subKey, "ImageMode", modeConstants, GetxRCNamedValue("AllLabels", "ImageMode", modeConstants, MULTIBLT_STRETCH)));

		hoverleftEdge = GetxRCInt(prefix, "HoverImageLeftEdge", GetxRCInt(subKey, "HoverImageLeftEdge", GetxRCInt("AllLabels", "HoverImageLeftEdge", leftEdge, 0), 0), 0);
		hovertopEdge = GetxRCInt(prefix, "HoverImageTopEdge", GetxRCInt(subKey, "HoverImageTopEdge", GetxRCInt("AllLabels", "HoverImageTopEdge", topEdge, 0), 0), 0);
		hoverrightEdge = GetxRCInt(prefix, "HoverImageRightEdge", GetxRCInt(subKey, "HoverImageRightEdge", GetxRCInt("AllLabels", "HoverImageRightEdge", rightEdge, 0), 0), 0);
		hoverbottomEdge = GetxRCInt(prefix, "HoverImageBottomEdge", GetxRCInt(subKey, "HoverImageBottomEdge", GetxRCInt("AllLabels", "HoverImageBottomEdge", bottomEdge, 0), 0), 0);

		hovermode = GetxRCNamedValue(prefix, "HoverImageMode", modeConstants, GetxRCNamedValue(subKey, "HoverImageMode", modeConstants, GetxRCNamedValue("AllLabels", "HoverImageMode", modeConstants, mode)));

		pressedleftEdge = GetxRCInt(prefix, "PressedImageLeftEdge", GetxRCInt(subKey, "PressedImageLeftEdge", GetxRCInt("AllLabels", "PressedImageLeftEdge", hoverleftEdge, 0), 0), 0);
		pressedtopEdge = GetxRCInt(prefix, "PressedImageTopEdge", GetxRCInt(subKey, "PressedImageTopEdge", GetxRCInt("AllLabels", "PressedImageTopEdge", hovertopEdge, 0), 0), 0);
		pressedrightEdge = GetxRCInt(prefix, "PressedImageRightEdge", GetxRCInt(subKey, "PressedImageRightEdge", GetxRCInt("AllLabels", "PressedImageRightEdge", hoverrightEdge, 0), 0), 0);
		pressedbottomEdge = GetxRCInt(prefix, "PressedImageBottomEdge", GetxRCInt(subKey, "PressedImageBottomEdge", GetxRCInt("AllLabels", "PressedImageBottomEdge", hoverbottomEdge, 0), 0), 0);

		pressedmode = GetxRCNamedValue(prefix, "PressedImageMode", modeConstants, GetxRCNamedValue(subKey, "PressedImageMode", modeConstants, GetxRCNamedValue("AllLabels", "PressedImageMode", modeConstants, hovermode)));
	
		int animimageWidth = 0;
		int animimageHeight = 0;
		imageFile = GetxRCString(prefix, "Frames", "");
		if (!imageFile.empty())
		{
			memoryAnimBitmap = LoadLSImage(imageFile.c_str(), 0);

			if (UseAlphaMap && memoryAnimBitmap)
			{
				BITMAP bm;
				GetObject(memoryAnimBitmap, sizeof(BITMAP), &bm);
				has_animAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

				if (has_animAlphaMap)
					PrepareAlphaChannel(memoryAnimBitmap);
			}

			GetLSBitmapSize(memoryAnimBitmap, &animimageWidth, &animimageHeight);

			memoryAnimDC = CreateCompatibleDC(0);
			SelectObject(memoryAnimDC, memoryAnimBitmap);

			frameWidth = GetxRCInt(prefix, "FrameWidth", 0);
			frameHeight = GetxRCInt(prefix, "FrameHeight", 0);
		}

		int hoveranimimageWidth = 0;
		int hoveranimimageHeight = 0;
		imageFile = GetxRCString(prefix, "HoverFrames", "");
		if (!imageFile.empty())
		{
			memoryHoverAnimBitmap = LoadLSImage( imageFile.c_str(), 0 );

			if (UseAlphaMap && memoryHoverAnimBitmap)
			{
				BITMAP bm;
				GetObject(memoryHoverAnimBitmap, sizeof(BITMAP), &bm);
				has_hoveranimAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

				if (has_hoveranimAlphaMap)
					PrepareAlphaChannel(memoryHoverAnimBitmap);
			}
			
			GetLSBitmapSize(memoryHoverAnimBitmap, &hoveranimimageWidth, &hoveranimimageHeight);
			
			memoryHoverAnimDC = CreateCompatibleDC(0);
			SelectObject(memoryHoverAnimDC, memoryHoverAnimBitmap);

			hoverframeWidth = GetxRCInt(prefix, "HoverFrameWidth", 0);
			hoverframeHeight = GetxRCInt(prefix, "HoverFrameHeight", 0);
		}
		
		if (!UseAlphaMap)
			paintEffects(memoryDC, imageWidth, imageHeight, 0);
		if (memoryHoverDC && !UseAlphaMap)
			paintEffects(memoryHoverDC, hoverimageWidth, hoverimageHeight, 1);
		if (memoryPressedDC && !UseAlphaMap)
			paintEffects(memoryPressedDC, pressedimageWidth, pressedimageHeight, 2);
		if (memoryAnimDC && !UseAlphaMap)
			paintEffects(memoryAnimDC, animimageWidth, animimageHeight, 0);
		if (memoryHoverAnimDC && !UseAlphaMap)
			paintEffects(memoryHoverAnimDC, hoveranimimageWidth, hoveranimimageHeight, 1);
	}
}

bool ImageTexture::apply(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType)
{
	if (imageType == 0 && memoryBitmap == 0 && hIcon == 0)
		return false;
	else if ( imageType == 1 && memoryHoverBitmap == 0 )
		imageType = 0;
	else if ( imageType == 2 && memoryPressedBitmap == 0 && memoryHoverBitmap != 0 )
		imageType = 1;
	else if ( imageType == 2 && memoryPressedBitmap == 0 && memoryHoverBitmap == 0 )
		imageType = 0;

	if (hIcon)
	{
		DrawIconEx(hDC, 0, 0, hIcon, width, height, 0, NULL, DI_NORMAL);

		return false;
	}
	else
	{
		if (transparencyMode == 2)
		{
			HDC bufferDC = CreateCompatibleDC(hDC);
			HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
			SelectObject(bufferDC, bufferBitmap);

			SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

			if (imageType == 0)
			{
				MultiBlt(bufferDC, 0, 0, width, height,
				memoryDC, 0, 0, imageWidth, imageHeight,
				leftEdge, topEdge, rightEdge, bottomEdge,
				mode, SRCCOPY);
			}
			else if (imageType == 1)
			{
				MultiBlt(bufferDC, 0, 0, width, height,
				memoryHoverDC, 0, 0, hoverimageWidth, hoverimageHeight,
				hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
				hovermode, SRCCOPY);
			}
			else if (imageType == 2)
			{
				MultiBlt(bufferDC, 0, 0, width, height,
				memoryPressedDC, 0, 0, pressedimageWidth, pressedimageHeight,
				pressedleftEdge, pressedtopEdge, pressedrightEdge, pressedbottomEdge,
				pressedmode, SRCCOPY);
			}
			
			TransparentBltLS(hDC, x, y, width, height,
				bufferDC, 0, 0,
				RGB(255, 0, 255));

			bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
		}
		else
		{
			SetStretchBltMode(hDC, STRETCH_DELETESCANS);

			if (imageType == 0)
			{
				MultiBlt(hDC, x, y, width, height,
					memoryDC, 0, 0, imageWidth, imageHeight,
					leftEdge, topEdge, rightEdge, bottomEdge,
					mode, SRCCOPY);
			}
			else if (imageType == 1)
			{
				MultiBlt(hDC, x, y, width, height,
					memoryHoverDC, 0, 0, hoverimageWidth, hoverimageHeight,
					hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
					hovermode, SRCCOPY);
			}
			else if (imageType == 2)
			{
				MultiBlt(hDC, x, y, width, height,
					memoryPressedDC, 0, 0, pressedimageWidth, pressedimageHeight,
					pressedleftEdge, pressedtopEdge, pressedrightEdge, pressedbottomEdge,
					pressedmode, SRCCOPY);
			}
		}

		if (imageType == 0)
			return has_AlphaMap;
		else if (imageType == 1)
			return has_hoverAlphaMap;
		else if (imageType == 2)
			return has_pressedAlphaMap;
		else 
			return false;
	}
}

bool ImageTexture::applyAnim(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType, int frameCounter)
{
	if (imageType == 0 && memoryAnimBitmap == 0)
		return false;
	if (imageType == 1 && memoryHoverAnimBitmap == 0)
		imageType = 0;

	if (transparencyMode == 2)
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		SelectObject(bufferDC, bufferBitmap);

		SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

		if (imageType == 0)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
				memoryAnimDC, (frameCounter-1)*frameWidth, 0, frameWidth, frameHeight,
				leftEdge, topEdge, rightEdge, bottomEdge,
				mode, SRCCOPY);
		}
		else if (imageType == 1)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
				memoryHoverAnimDC, (frameCounter-1)*hoverframeWidth, 0, hoverframeWidth, hoverframeHeight,
				hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
				hovermode, SRCCOPY);
		}

		TransparentBltLS(hDC, x, y, width, height,
			bufferDC, 0, 0,
			RGB(255, 0, 255));

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else
	{
		SetStretchBltMode(hDC, STRETCH_DELETESCANS);

		if (imageType == 0)
		{
			MultiBlt(hDC, x, y, width, height,
					memoryAnimDC, (frameCounter-1)*frameWidth, 0, frameWidth, frameHeight,
					leftEdge, topEdge, rightEdge, bottomEdge,
					mode, SRCCOPY);
		}
		else if (imageType == 1)
		{
			MultiBlt(hDC, x, y, width, height,
					memoryHoverAnimDC, (frameCounter-1)*hoverframeWidth, 0, hoverframeWidth, hoverframeHeight,
					hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
					hovermode, SRCCOPY);
		}
	}

	if (imageType == 0)
		return has_animAlphaMap;
	else if (imageType == 1)
		return has_hoveranimAlphaMap;
	else
		return false;
}

void ImageTexture::PrepareAlphaChannel(HBITMAP &bmp)
{
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for (int i = 0; i < bm.bmHeight * bm.bmWidth; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//Prepare Text
		if (pPixel[3] == 0)
			pPixel[3] = 1;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		//AlphaMap Colorizing
		//Saturation Effect
		/*if( _saturation != 255 )
		{
			BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);
			r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
			g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
			b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
		}
		// Hue effect
		if( _hueIntensity != 0 )
		{
			// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
			// test normal
			r = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
			b = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
		}*/
		
		//PreMultiply
		pPixel[0] = r * pPixel[3] / 255;
		pPixel[1] = g * pPixel[3] / 255;
		pPixel[2] = b * pPixel[3] / 255;

		pPixel+= 4;
	}
}

void ImageTexture::paintEffects(HDC destHDC, int width, int height, int imageType)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int _saturation = 255;
	COLORREF _hueColor = RGB(0,0,0);
	int _hueIntensity = 0;

	if (imageType == 0)
	{
		_saturation = saturation;
		_hueColor = hueColor;
		_hueIntensity = hueIntensity;
	}
	else if (imageType == 1)
	{
		_saturation = hoversaturation;
		_hueColor = hoverhueColor;
		_hueIntensity = hoverhueIntensity;
	}
	else if (imageType == 2)
	{
		_saturation = pressedsaturation;
		_hueColor = pressedhueColor;
		_hueIntensity = pressedhueIntensity;
	}

	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
			

			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				// Saturation effect
				if(_saturation != 255)
				{
					BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

					r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
					g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
					b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
				}

				// Hue effect
				if(_hueIntensity != 0)
				{
					// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
					// test normal
					r = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
				}
			}
			else 
				transparencyNeeded = true;

			SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}