#include "StdAfx.h"

#include "Label.h"
#include "LabelSettings.h"
#include "xFont.h"
#include "Texture.h"
#include "SystemInfo.h"
#include "ShowStyle.h"

#ifdef xLabelLight
//----------------------------------------------------------------
#include "shlobj.h"
//----------------------------------------------------------------
#endif

#ifndef xLabelLight
//----------------------------------------------------------------
	#include "ContextMenu.h"
//----------------------------------------------------------------
#endif

#define TIMER_MOUSETRACK 1
#define TIMER_UPDATE 2
#define TIMER_SCROLL 3
#define TIMER_AUTOHIDE 4
#define TIMER_MOUSETRACK_MOVE 5
#define TIMER_ALPHAFADEIN 6
#define TIMER_ALPHAFADEOUT 7
#define TIMER_FRAMEANIMATION 8
#define TIMER_SOLIDPULSEANIMATION 9

extern LabelList labelList;

#ifndef xLabelLight
//----------------------------------------------------------------
	extern LabelList desktoplabelList;
	extern LabelList shortcutlabelList;
//----------------------------------------------------------------
#endif

Label::Label(const string &name):actualheight(0),actualwidth(0),actualx(0),actualy(0)
{
	this->name = name;

	hWnd = 0;
	hInstance = 0;
	box = 0;

	TooltipHints = 0;
	cursor = 0;
	handcursor = 0;

	labelType = -1;

	background = 0;
	font = 0;

	borderMethod = 0;
	hoverborderMethod = 0;
	pressedborderMethod = 0;

	backgroundDC = 0;
	bufferDC = 0;
	backgroundBitmap = 0;
	bufferBitmap = 0;

	hoverActive = false;
	pressedActive = false;

	useHover = false;
	usePressed = false;
	
	hoverSound = "";
	pressedSound = "";

	focusOnEnter = 0;

	mousePressed = false;
	mouseInside = false;

	mouseInsideHREF = false;

	movemodifierkeyPressed = false;
	moveButtonPressed = 0;
	moveButton = 0;

	hasMoved = false;

	visible = false;

	bUsingDefSkin = false;

	noTextSplit = false;
	text = "";
	originalText.push_back("");
	originalTextBackup = "";
	hrefconfig.push_back("");
	current = 0;
	currentBackup = 0;

	dynamicText = false;
	dynamicTooltip = false;

	autoWidthMode = 0;
	autoHeightMode = 0;

	scroll = 0;
	scrollBackup = 1;
	scrollLimit = 0;
	scrollPosition = 0;
	frozen = false;
	
	//Animations
	frameCounter = 0;
	loopCounter = -1;
	normalAnim = false;
	hoverAnim = false;

	transparencyNeeded = true;
	
	noEscape = false;

	layeredFlag = false;
	UpdateAlpha = 0;

	alphaFadeStep = 2;
	alphaFadeDelay = 10;

	startHidden = false;

	lastImageMode = 0;
	overblendStyle = 0;

	DISnapTo = false;

	#ifndef xLabelLight
	//----------------------------------------------------------------
		rememberPosition = false;
		easyDesktopFocus = false;
	//----------------------------------------------------------------
	#endif
}

Label::~Label()
{
	if (cursor && !cursorPath.empty())
		DestroyCursor(cursor);
	if (handcursor)
		DestroyCursor(handcursor);

	if (hWnd && TooltipHints)
	{
		removeHint(hWnd);
		DestroyWindow(TooltipHints);
		TooltipHints = 0;
	}

	if(IsWindow(hWnd))
	{
		KillTimer(hWnd, TIMER_MOUSETRACK_MOVE);
		KillTimer(hWnd, TIMER_MOUSETRACK);
		KillTimer(hWnd, TIMER_UPDATE);
		KillTimer(hWnd, TIMER_SCROLL);
		KillTimer(hWnd, TIMER_AUTOHIDE);
		KillTimer(hWnd, TIMER_ALPHAFADEIN);
		KillTimer(hWnd, TIMER_ALPHAFADEOUT);
		KillTimer(hWnd, TIMER_FRAMEANIMATION);
		KillTimer(hWnd, TIMER_SOLIDPULSEANIMATION);

		DestroyWindow(hWnd);
	}

	if(backgroundBitmap)
	{
		backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
		DeleteObject(backgroundBitmap);
	}
	if (backgroundDC)
		DeleteDC(backgroundDC);

	if(bufferBitmap)
	{
		bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
		DeleteObject(bufferBitmap);
	}
	if (bufferDC)
		DeleteDC(bufferDC);

	if( background && (!bUsingDefSkin || labelType == 1 || labelType == 2) )
		delete background;

	if( font && (font != defaultSettings->font || labelType == 1 || labelType == 2) )
		delete font;
}

void Label::load(HINSTANCE hInstance, HWND box, bool dropShadow)
{
	this->hInstance = hInstance;
	
	if (IsWindow(box))
	{
		this->box = box;
	}
	else
	{
		this->box = NULL;
	}
	
	if (IsWindow(hWnd))	// window already exists...
		return;

	if(!IsOS(OS_XP))
		dropShadow = false;

	hWnd = CreateWindowEx( (box ? 0 : WS_EX_TOOLWINDOW),
		(dropShadow ? "xLabelXP" : "xLabel"),
		name.c_str(),
		(box ? WS_CHILD : WS_POPUP) | WS_CLIPCHILDREN,
		actualx, actualy,		
		actualwidth, actualheight,
		box ? box : GetLitestepWnd(), 
		0,
		hInstance,
		this);
		
	if (hWnd)
	{
		SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

		reconfigure();

		repaint(true);
	}
	else
		delete this;
}

void Label::reconfigure(bool refresh)
{
	dynamicText = false;
	dynamicTooltip = false;

	LabelSettings settings(name.c_str(), box);
	
	//Extra Settings
	labelGroup = settings.labelGroup;
	bUseFahrenheit = settings.bUseFahrenheit;

	//Cursor
	cursorPath = settings.cursorPath;

	//Module Hook
	if (!refresh)
		moduleHook = GetxRCLineList( ("*" + name + "ModuleHook").c_str() );

	//Normal Label = Type 0
	labelType = 0;

	//Painting Background
	char normal[32], hover[32], pressed[32];
	char *buffers[] = {normal, hover, pressed};

	int numTokens = LCTokenize(settings.borderMethod.c_str(), buffers, 3, NULL);
		
	if(numTokens >= 1)
	{	
		if (stricmp(normal, "raised") == NULL)
			borderMethod = 1;
		else if (stricmp(normal, "sunken") == NULL)
			borderMethod = 2;
		else if (stricmp(normal, "etched") == NULL)
			borderMethod = 3;
		else if (stricmp(normal, "bump") == NULL)
			borderMethod = 4;
		else 
			borderMethod = 0;
	}
	else
		borderMethod = 0;
	if (numTokens == 2)
	{
		if (stricmp(hover, "raised") == NULL)
			hoverborderMethod = 1;
		else if (stricmp(hover, "sunken") == NULL)
			hoverborderMethod = 2;
		else if (stricmp(hover, "etched") == NULL)
			hoverborderMethod = 3;
		else if (stricmp(hover, "bump") == NULL)
			hoverborderMethod = 4;
		else 
			hoverborderMethod = borderMethod;
	}
	else if (numTokens == 3)
	{
		if (stricmp(hover, "raised") == NULL)
			hoverborderMethod = 1;
		else if (stricmp(hover, "sunken") == NULL)
			hoverborderMethod = 2;
		else if (stricmp(hover, "etched") == NULL)
			hoverborderMethod = 3;
		else if (stricmp(hover, "bump") == NULL)
			hoverborderMethod = 4;
		else 
			hoverborderMethod = borderMethod;

		if (stricmp(pressed, "raised") == NULL)
			pressedborderMethod = 1;
		else if (stricmp(pressed, "sunken") == NULL)
			pressedborderMethod = 2;
		else if (stricmp(pressed, "etched") == NULL)
			pressedborderMethod = 3;
		else if (stricmp(pressed, "bump") == NULL)
			pressedborderMethod = 4;
		else 
			pressedborderMethod = hoverborderMethod;
	}
	else
	{
		hoverborderMethod = borderMethod;
		pressedborderMethod = borderMethod;
	}

	setBackground(settings.skin);	
	transparencyNeeded = background->isTransparencyNeeded();

	//Transparency Settings
	transparencyMode = settings.transparencyMode;
	alphaTransparency = settings.alphaTransparency;
	alphaMap = settings.alphaMap;
	if (alphaMap)
		alphaMap = background->isAlphaMapNeeded();

	alphaFade = settings.alphaFade;

	char step[32], time[32];
	char *alphabuffers[] = {step, time};

	numTokens = 0;
	numTokens = LCTokenize(settings.customalphaFade.c_str(), alphabuffers, 2, NULL);

	if (numTokens == 1)
	{
		alphaFadeStep = atoi( step );
		if (alphaFadeStep < 1 || alphaFadeStep > 255)
			alphaFadeStep = 2;
	}
	else if (numTokens == 2)
	{
		alphaFadeStep = atoi( step );
		if (alphaFadeStep < 1 || alphaFadeStep > 255)
			alphaFadeStep = 2;
		alphaFadeDelay = atoi( time );
		if (alphaFadeDelay < 1 || alphaFadeDelay > 500)
			alphaFadeDelay = 10;
	}

	if ( (alphaTransparency != 255 || alphaMap || alphaFade) && IsOS(OS_2KXP) && box == NULL)
	{
		if ( (GetxRCString(name.c_str(), "Image", "")).empty() && (GetxRCString(labelGroup.c_str(), "Image", "")).empty() && (GetxRCString("AllLabels", "Image", "")).empty())
			alphaMap = false;
	}
	else
	{
		alphaTransparency = 255;
		alphaMap = false;
		alphaFade = false;
	}

	//AutoSize
	autoWidthMode = settings.autoWidthMode;
	autoHeightMode = settings.autoHeightMode;
	autoMinWidth = settings.width;
	autoMinHeight = settings.height;
	autoMaxWidth = settings.autoMaxWidth;
	autoMaxHeight = settings.autoMaxHeight;
	if (!refresh)
	{
		startCenterX = settings.x+(settings.width/2);
		startCenterY = settings.y+(settings.height/2);
	}

	//Hover/Pressed Images
	if ( !(GetxRCString(name.c_str(), "HoverImage", "")).empty() || !(GetxRCString(labelGroup.c_str(), "HoverImage", "")).empty() || !(GetxRCString("AllLabels", "HoverImage", "")).empty() ||
			!(GetxRCString(name.c_str(), "HoverFrames", "")).empty() || !(GetxRCString(labelGroup.c_str(), "HoverFrames", "")).empty() || !(GetxRCString("AllLabels", "HoverFrames", "")).empty() ||
			(hoverborderMethod != 0 && hoverborderMethod != borderMethod) || 
			!(GetxRCString(name.c_str(), "SolidHoverColors", "")).empty() || !(GetxRCString(labelGroup.c_str(), "SolidHoverColors", "")).empty() || !(GetxRCString("AllLabels", "SolidHoverColors", "")).empty() )
		useHover = true;
	
	if ( !(GetxRCString(name.c_str(), "PressedImage", "")).empty() || !(GetxRCString(labelGroup.c_str(), "PressedImage", "")).empty() || !(GetxRCString("AllLabels", "PressedImage", "")).empty() ||
			(pressedborderMethod != 0 && pressedborderMethod != hoverborderMethod && pressedborderMethod != borderMethod) || 
			!(GetxRCString(name.c_str(), "SolidPressedColors", "")).empty() || !(GetxRCString(labelGroup.c_str(), "SolidPressedColors", "")).empty() || !(GetxRCString("AllLabels", "SolidPressedColors", "")).empty())
		usePressed = true;

	//Animations
	overblendStyle = settings.overblendStyle;

	imageLoop = GetxRCInt(name, "FrameLoops", -1);
	imageFrameDelay = GetxRCInt(name, "FrameDelay", 100, 1);
	imageFrameCount = GetxRCInt(name, "FrameCount", 0, 0);
	if (imageFrameCount != 0)
		normalAnim = true;
	hoverimageLoop = GetxRCInt(name, "HoverFrameLoops", -1);
	hoverimageFrameDelay = GetxRCInt(name, "HoverFrameDelay", 100, 1);
	hoverimageFrameCount = GetxRCInt(name, "HoverFrameCount", 0, 0);
	if (hoverimageFrameCount != 0)
		hoverAnim = true;

	solidPulseDelay = GetxRCInt(name, "SolidPulseDelay", 0, 0, 1000);

	//Painting Text
	setFont(settings.font);			
	align = settings.align;
	vertAlign = settings.vertAlign;
	leftBorder = settings.leftBorder;
	topBorder = settings.topBorder;
	rightBorder = settings.rightBorder;
	bottomBorder = settings.bottomBorder;
	noEscape = settings.noEscape;
	//Must set BEFORE setText to work
	noTextSplit = settings.noTextSplit;
	if (!refresh)
		setText(settings.text, true);

	hrefconfig = split(settings.hrefconfig, ";");

	//Scrolling
	scrollPadLength = settings.scrollPadLength;
	scrollInterval = settings.scrollInterval;
	scrollSpeed = settings.scrollSpeed;
	setScrolling(settings.scroll);

	//Updates
	alwaysUpdateContent = settings.alwaysUpdateContent;
	updateInterval = settings.updateInterval;

	//Events/Actions
	leftClickCommand = settings.leftClickCommand;
	leftClickDownCommand = settings.leftClickDownCommand;
	leftDoubleClickCommand = settings.leftDoubleClickCommand;
	middleClickCommand = settings.middleClickCommand;
	middleClickDownCommand = settings.middleClickDownCommand;
	middleDoubleClickCommand = settings.middleDoubleClickCommand;
	rightClickCommand = settings.rightClickCommand;
	rightClickDownCommand = settings.rightClickDownCommand;
	rightDoubleClickCommand = settings.rightDoubleClickCommand;
	wheelDownCommand = settings.wheelDownCommand;
	wheelUpCommand = settings.wheelUpCommand;
	enterCommand = settings.enterCommand;
	leaveCommand = settings.leaveCommand;
	dropCommand = settings.dropCommand;
	textChangeCommand = settings.textChangeCommand;
	resizeCommand = settings.resizeCommand;
	moveCommand = settings.moveCommand;

	hideCommand = settings.hideCommand;
	showCommand = settings.showCommand;

	//Event Regions
	labelLeftClickRegions = GetxRCLineList( ("*" + name + "OnLeftClickRegion").c_str() );
	labelRightClickRegions = GetxRCLineList( ("*" + name + "OnRightClickRegion").c_str() );
	labelMiddleClickRegions = GetxRCLineList( ("*" + name + "OnMiddleClickRegion").c_str() );

	//Snap To Settings
	char distance[16], what[64];
	char *snapbuffers[] = {distance, what};

	numTokens = LCTokenize(settings.snapTo.c_str(), snapbuffers, 2, NULL);
	snapDistance = atoi(distance);
	if (numTokens == 1)
		snapTo = "desktop";
	else
		snapTo = what;

	if (settings.focusOnEnter)
		focusOnEnter = 1;
	else
		focusOnEnter = 0;

	//Moving
	moveable = settings.moveable;
	moveKey = settings.moveKey;
	moveButton = settings.moveButton;
		
	if (hWnd)
	{
		if (cursor)
		{
			DestroyCursor(cursor);
			cursor = 0;
		}
		
		SendMessage(hWnd, LM_SETCURSOR, 0, 0);

		if ( IsOS(OS_2KXP) )
			ghosted = settings.ghosted;
		else
			ghosted = false;

		if( alphaTransparency != 255 || alphaMap || alphaFade || ghosted)
		{
			if (!refresh)
			{
				// MUST BE SET BEFORE setAlwaysOnTop()
				ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);
				
				setGhosted(ghosted);

				layeredFlag = true;

				if (alphaFade)
					UpdateLabelAlpha(0, true);
				else
					UpdateLabelAlpha(alphaTransparency, true);
			}
			else if (refresh)
				setGhosted(ghosted);
		}

		//File Actions
		if (dropCommand.empty())
			DragAcceptFiles(hWnd, false);
		else
			DragAcceptFiles(hWnd, true);

		//AlwaysOnTop
		if (!refresh)
			setAlwaysOnTop(settings.alwaysOnTop);

		startHidden = settings.startHidden;
	}

	//Initial Repositioning
	if (!refresh)
	{
		reposition(settings.x, settings.y, settings.width, settings.height);

		if (autoWidthMode != 0 || autoHeightMode != 0)
			autoSize();
	}

	//ToolTips
	balloontooltip = settings.balloontooltip;
	originalTooltip = settings.tooltip;

	tooltipmaxwidth = settings.tooltipmaxwidth;
	if (!originalTooltip.empty())
	{
		while ( strstr(originalTooltip.c_str(), "\\n") != NULL )
		{
			originalTooltip.replace(originalTooltip.find("\\n"), 2, "\n", 1);
		}
		while ( strstr(originalTooltip.c_str(), "<br>") != NULL )
		{
			originalTooltip.replace(originalTooltip.find("<br>"), 4, "\n", 1);
		}

		if (TooltipHints == 0)
		{
			TooltipHints = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP | (balloontooltip ? TTS_BALLOON : 0),
										  CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
										  NULL, NULL, hInstance, NULL);
			if (TooltipHints != 0)
			{
				SetWindowPos(TooltipHints, HWND_TOPMOST, 0, 0, 0, 0,
							 SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

				SendMessage(TooltipHints, TTM_SETMAXTIPWIDTH, 0, tooltipmaxwidth);

				strcpy(tooltip, (systemInfo->processLabelText(originalTooltip, this, &dynamicTooltip)).c_str());
				addHint(hWnd, tooltip);
			}
		}
		else
		{
			strcpy(tooltip, (systemInfo->processLabelText(originalTooltip, this, &dynamicTooltip)).c_str());
			updateHint(hWnd, tooltip);
		}
	}
	else
	{
		if (hWnd && TooltipHints != 0)
		{
			removeHint(hWnd);
			DestroyWindow(TooltipHints);
			TooltipHints = 0;
		}
	}

	if (refresh && bufferDC)
		onPaint(bufferDC);

	//Update Region (Magic Pink)
	if (transparencyMode == 1)
		updateRegion();
	
	update();

	if (solidPulseDelay != 0 && !normalAnim)
		SetTimer(hWnd, TIMER_SOLIDPULSEANIMATION, solidPulseDelay, NULL);
	else if (normalAnim)
		SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
	else
	{
		KillTimer(hWnd, TIMER_FRAMEANIMATION);
		KillTimer(hWnd, TIMER_SOLIDPULSEANIMATION);
	}
}

#ifndef xLabelLight
//----------------------------------------------------------------
	void Label::desktopload(HINSTANCE hInstance, string configLine)
	{
		this->hInstance = hInstance;
		
		this->box = NULL;
		
		if (IsWindow(hWnd))	// window already exists...
			return;

		hWnd = CreateWindowEx(WS_EX_TOOLWINDOW,
			"xLabel",
			name.c_str(),
			WS_POPUP,
			actualx, actualy,			
			actualwidth, actualheight,
			GetLitestepWnd(),  
			0,
			hInstance,
			this);

		if (hWnd)
		{
			SetWindowLong(hWnd, GWL_USERDATA, magicDWord);
			
			//ToolTips for Icons
			TooltipHints = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP,
										  CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
										  NULL, NULL, hInstance, NULL);
			if (TooltipHints)
			{
				SetWindowPos(TooltipHints, HWND_TOPMOST, 0, 0, 0, 0,
							 SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

				SendMessage(TooltipHints, TTM_SETMAXTIPWIDTH, 0, 600);
			}

			desktopreconfigure(configLine);
		}
		else
			delete this;
	}

	void Label::desktopreconfigure(string configLine)
	{
		NameValuePair iconTextValues[] = {
			{ "top", 1 },
			{ "bottom", 2 },
			{ "left", 3 },
			{ "right", 4 },
			{ 0, 0 }
		};

		//Cursor
		cursorPath = GetxRCString("xlabeldesktop", "Cursor", "");

		//DesktopLabel = Type 1
		labelType = 1;

		//Transparency Settings
		transparencyMode = 1;
		alphaTransparency = GetxRCInt("xlabeldesktop", "AlphaTransparency", 255, 0 , 255);
		alphaMap = false;
		alphaFade = false;

		if ( !IsOS(OS_2KXP) )
			alphaTransparency = 255;

		//Painting Background

		//Prepare Settings
		//----------------------------------------------------------------
		char xname[32], ix[16], iy[16], path[MAX_PATH_LENGTH];
		char *buffers[] = {xname, ix, iy, path};

		LCTokenize(configLine.c_str(), buffers, 4, NULL);
		
		actualx = xParseCoord(ix, 0, GetSystemMetrics(SM_CXSCREEN));
		actualy = xParseCoord(iy, 0, GetSystemMetrics(SM_CXSCREEN));
		actualwidth = GetxRCInt("xlabeldesktop", "ItemWidth", 100);
		actualheight = GetxRCInt("xlabeldesktop", "ItemHeight", 75);
		
		//----------------------------------------------------------------
		//SnapTo Settings
		char diX[64], diY[64], diWidth[64], diHeight[64], diDistance[64];
		char *DIbuffers[] = {diX, diY, diWidth, diHeight, diDistance};

		if ( LCTokenize( (GetxRCLine("xlabeldesktop", "SnapTo", "")).c_str(), DIbuffers, 5, NULL) == 5 )
		{
			DISnapTo = true;
			DIx = atoi(diX);
			DIy = atoi(diY);
			DIwidth = atoi(diWidth);
			DIheight = atoi(diHeight);
			snapDistance = atoi(diDistance);
		}

		setBackground( GetDesktopTexture(path, this->name) );	
		transparencyNeeded = true;

		//AutoSize
		autoWidthMode = 0;
		autoHeightMode = 0;

		//Hover/Pressed Images
		useHover = false;
		usePressed = false;

		//Animations
		normalAnim = false;
		hoverAnim = false;

		//Prepare Special DesktopSettings
		//----------------------------------------------------------------
		configSavePath = GetxRCString("xlabeldesktop", "ConfigFile", "");
		rememberPosition = GetxRCBool("xlabeldesktop", "RememberPosition", true);

		int iconSize = GetxRCInt("xlabeldesktop", "IconSize", 32);
		int iconTextAlign = GetxRCNamedValue("xlabeldesktop", "TextPosition", iconTextValues, 2);
		int iconTextSpacing = GetxRCInt("xlabeldesktop" , "TextSpacing", 5);
		//----------------------------------------------------------------

		//Painting Text
		setFont( GetRCFont("xlabeldesktop") );

		if (iconTextAlign == 1 || iconTextAlign == 2)
		{
			leftBorder = 0;
			rightBorder = 0;
			align = DT_CENTER;
		}
		else if (iconTextAlign == 3)
		{
			leftBorder = 0;
			rightBorder = iconSize+iconTextSpacing+1;
			align = DT_RIGHT;
		}
		else if (iconTextAlign == 4)
		{
			leftBorder = iconSize+iconTextSpacing+1;
			rightBorder = 0;
			align = DT_LEFT;
		}

		if (iconTextAlign == 1)
		{
			topBorder = 0;
			bottomBorder = iconSize+iconTextSpacing+1;
			vertAlign = ALIGN_BOTTOM;
		}
		else if (iconTextAlign == 2)
		{
			topBorder = iconSize+iconTextSpacing+1;
			bottomBorder = 0;
			vertAlign = ALIGN_TOP;
		}
		else if (iconTextAlign == 3 || iconTextAlign == 4)
		{
			topBorder = 0;
			bottomBorder = 0;
			vertAlign = ALIGN_CENTER;
		}

		//Must be set BEFORE setText to work
		noTextSplit = true;

		iconCommand = path;
		bool itemIsFolder = false;
		
		HRESULT hres;
		IShellLink* psl;
		SHFILEINFO shFileInfo;

		char szGetPath[MAX_PATH_LENGTH];
		if (strstr(iconCommand.c_str(), "::{") != NULL)
		{
			itemIsFolder = true;
			SHGetFileInfo(iconCommand.c_str(), NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_DISPLAYNAME | SHGFI_TYPENAME);
			setText( shFileInfo.szDisplayName );
			strcpy(tooltip , "Type:    ");
			strcat(tooltip , shFileInfo.szTypeName);
			strcat(tooltip, "\nTarget: ");
			strcat(tooltip , shFileInfo.szDisplayName);
		}
		else
		{
			SHGetFileInfo(iconCommand.c_str(), NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_DISPLAYNAME | SHGFI_TYPENAME | SHGFI_ATTRIBUTES);
			setText( shFileInfo.szDisplayName );
			strcpy(tooltip , "Type:    ");
			strcat(tooltip , shFileInfo.szTypeName);
		
			if (shFileInfo.dwAttributes & SFGAO_FOLDER)
				itemIsFolder = true;
		}

		//Create the ShellLink object
		hres = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID*) &psl);

		if (SUCCEEDED(hres))
		{
			IPersistFile* ppf;
			//Bind the ShellLink object to the Persistent File
			hres = psl->QueryInterface( IID_IPersistFile, (LPVOID *) &ppf);

			if (SUCCEEDED(hres))
			{
				WORD wsz[MAX_PATH_LENGTH];
				//Get a UNICODE wide string wsz from the Link path
				MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, path, -1, wsz, MAX_PATH_LENGTH);
					
				//Read the link into the persistent file
				hres = ppf->Load(wsz, 0);
					
				if (SUCCEEDED(hres))
				{
					//Read the target information from the link object
					//UNC paths are supported (SLGP_UNCPRIORITY)
					psl->GetPath(szGetPath, MAX_PATH_LENGTH, NULL, SLGP_UNCPRIORITY);
						
					strcat(tooltip, "\nTarget: ");
					strcat(tooltip, szGetPath);

					psl->GetDescription(szGetPath, MAX_PATH_LENGTH);

					if (stricmp(szGetPath, "") != NULL)
					{
						strcat(tooltip, "\n\nDescription:\n");
						strcat(tooltip, szGetPath);

						SendMessage(TooltipHints, TTM_SETMAXTIPWIDTH, 0, 300);
					}
				}
				else if (strstr(path, "::{") == NULL)
				{
					strcat(tooltip, "\nName:   ");
					strcat(tooltip, iconCommand.c_str());
				}
			}
		}
		psl->Release();

		addHint(hWnd, tooltip);

		//Scrolling
		scrollPadLength = 10;
		scrollInterval = 100;
		scrollSpeed = 1;
		setScrolling(0);

		//Updates
		alwaysUpdateContent = false;
		updateInterval = 1000;

		//Extra Settings
		labelGroup = "xlabeldesktop";
		bUseFahrenheit = false;

		//Events/Actions
		easyDesktopFocus = GetxRCBool("xlabeldesktop", "EasyFocus", false);
		
		leftClickDownCommand = "";
		leftClickCommand = GetxRCString("xlabeldesktop", "OnLeftClick", ".move");
		leftDoubleClickCommand  = GetxRCString("xlabeldesktop", "OnLeftDoubleClick", ".open");
		middleClickDownCommand = "";
		middleClickCommand = GetxRCString("xlabeldesktop", "OnMiddleClick", ".none");
		middleDoubleClickCommand = GetxRCString("xlabeldesktop", "OnMiddleDoubleClick", ".none");
		rightClickDownCommand = "";
		rightClickCommand =  GetxRCString("xlabeldesktop", "OnRightClick", ".context");
		rightDoubleClickCommand =  GetxRCString("xlabeldesktop", "OnRightDoubleClick", ".none");

		if (itemIsFolder)
		{
			if (strstr(iconCommand.c_str(), "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}") != NULL)
			{
				dropCommand = "";
				itemIsFolder = false;
			}
			else if (strstr(iconCommand.c_str(), "::{450D8FBA-AD25-11D0-98A8-0800361B1103}") != NULL)
			{
				SHGetSpecialFolderPathA(NULL, path, CSIDL_PERSONAL, false);
				dropCommand = "copy \"";
				dropCommand.append( path );
				dropCommand.append( "\"" );
			}
			else if (strstr(iconCommand.c_str(), "::{208D2C60-3AEA-1069-A2D7-08002B30309D}") != NULL)
			{
				dropCommand = "";
				itemIsFolder = false;
			}
			else if (strstr(iconCommand.c_str(), "::{645FF040-5081-101B-9F08-00AA002F954E}") != NULL)
			{
				dropCommand = "delete files";
			}
			else
				dropCommand = ("copy \"" + iconCommand + "\"");
		}
		else
		{
			dropCommand = "";
		}

		//Moving
		moveable = GetxRCBool("xlabeldesktop", "Moveable", true);
		moveKey = -1;
		//Apply Selection of MoveButton
		if (stricmp(leftClickCommand.c_str(), ".move") == NULL)
			moveButton = 1;

		if (stricmp(middleClickCommand.c_str(), ".move") == NULL)
			moveButton = 3;

		if (stricmp(rightClickCommand.c_str(), ".move") == NULL)
			moveButton = 2;

		if (moveButton == 0)
			moveable = false;

		//Apply Selection of clicks
		if (stricmp(leftClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			leftClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(leftClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			leftClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else if (stricmp(leftClickCommand.c_str(), ".context") != NULL && stricmp(leftClickCommand.c_str(), ".properties") != NULL)
			leftClickCommand = "";

		if (stricmp(leftDoubleClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			leftDoubleClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(leftDoubleClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			leftDoubleClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else
			leftDoubleClickCommand = "";

		if (stricmp(middleClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			middleClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(middleClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			middleClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else if (stricmp(middleClickCommand.c_str(), ".context") != NULL && stricmp(middleClickCommand.c_str(), ".properties") != NULL)
			middleClickCommand = "";
		
		if (stricmp(middleDoubleClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			middleDoubleClickCommand = ("!execute[\"" + iconCommand + "\"]");
		else if (stricmp(middleDoubleClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			middleDoubleClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else
			middleDoubleClickCommand = "";

		if (stricmp(rightClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			rightClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(rightClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			rightClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else if (stricmp(rightClickCommand.c_str(), ".context") != NULL && stricmp(rightClickCommand.c_str(), ".properties") != NULL)
			rightClickCommand = "";
		
		if (stricmp(rightDoubleClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			rightDoubleClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(rightDoubleClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			rightDoubleClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else
			rightDoubleClickCommand = "";

		wheelDownCommand = "";
		wheelUpCommand = "";
		enterCommand = "";
		leaveCommand = "";
		textChangeCommand = "";
		resizeCommand = "";
		moveCommand = "";

		hideCommand = "";
		showCommand = "";

		//Event Regions
		labelLeftClickRegions.clear();
		labelRightClickRegions.clear();
		labelMiddleClickRegions.clear();

		labelHREFClickRegions.clear();

		//Initial Repositioning
		reposition(actualx, actualy , actualwidth, actualheight);
		
		if (hWnd)
		{
			SendMessage(hWnd, LM_SETCURSOR, 0, 0);

			if( alphaTransparency != 255 )
			{
				// MUST BE SET BEFORE setAlwaysOnTop()
				ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

				layeredFlag = true;

				UpdateLabelAlpha(alphaTransparency, true);
			}

			setGhosted(false);

			//File Actions
			if (itemIsFolder)
				DragAcceptFiles(hWnd, true);
			else
				DragAcceptFiles(hWnd, false);

			//AlwaysOnTop
			setAlwaysOnTop(false);	

			//Update Region (Magic Pink)
			updateRegion();

			//Initial Show
			GetxRCBool("xlabeldesktop", "StartHidden") ? hide() : show();
		}

		tooltipmaxwidth = 300;

		update();
	}

	void Label::shortcutload(HINSTANCE hInstance, string configLine, int lastx, int lasty, HWND box)
	{
		this->hInstance = hInstance;
		
   		if (IsWindow(box))
		{
			this->box = box;
		}
		else
		{
			this->box = NULL;
		}
		
		if (IsWindow(hWnd))	// window already exists...
			return;
			
		hWnd = CreateWindowEx(box ? 0 : WS_EX_TOOLWINDOW,
			"xLabel",
			name.c_str(),
			box ? WS_CHILD : WS_POPUP,
			actualx, actualy,		
			actualwidth, actualheight,	
			box ? box : GetLitestepWnd(), 
			0,
			hInstance,
			this);

		if (hWnd)
		{
			SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

			//ToolTips for Shortcut2
			if ( shortcutreconfigure(configLine, lastx, lasty) )
			{	
				TooltipHints = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP,
										  CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
										  NULL, NULL, hInstance, NULL);
				
				if (TooltipHints)
					SetWindowPos(TooltipHints, HWND_TOPMOST, 0, 0, 0, 0,
							 SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

				strcpy(tooltip, name.c_str());
				addHint(hWnd, tooltip);
			}
		}
		else
			delete this;
	}

	bool Label::shortcutreconfigure(string configLine, int lastx, int lasty)
	{
		//Cursor
		cursorPath = "";

		//ShortcutLabel = Type 2
		labelType = 2;

		alphaTransparency = 255;
		alphaMap = false;
		alphaFade = false;

		//Prepare & Process Settings
		//----------------------------------------------------------------
		//Initializing
		LPSTR fLCommand = "";
		LPSTR fRCommand = "";
		LPSTR fMCommand = "";
		LPSTR fDropCommand = "";
		
		char name[32], ix[16], iy[16], normal[64], hover[64], pressed[64], rest[MAX_LINE_LENGTH];
		char *buffers[] = {name, ix, iy, normal, hover, pressed};

		configLine = lowerCase(configLine);
		LCTokenize(configLine.c_str(), buffers, 6, rest);

		//Hover/Pressed Images
		if ( stricmp(hover, ".none") != 0 )
			useHover = true;
		if ( stricmp(pressed, ".none") != 0 )
			usePressed = true;
		
		bool IsVisible;
		bool AcceptInput;
		bool Transparency;
		bool UseTooltip = GetxRCBool("shortcut", "ShowCaptions", true);

		labelGroup = "0";
		char token[MAX_LINE_LENGTH];
		LPCSTR nextToken = rest;
		bool hasCommand = true;

		if (GetToken(nextToken, token, &nextToken, false))
		{
			if (token[0] == '#')
			{
				int tmpGroup = atoi(&token[1]);	
				labelGroup = IntToString(tmpGroup);
				AcceptInput = !strchr(token, 'k');
				IsVisible = !strchr(token, 'h');
				alwaysOnTop = strchr(token, 't') != 0;
				Transparency = !strchr(token, 'o') != 0;
				moveable = strchr(token, 'm') != 0;
				rememberPosition = strchr(token, 's') != 0;
				alphaMap = strchr(token, 'a') != 0;

				if (UseTooltip)
					UseTooltip = !strchr(token, 'c');
				else
					UseTooltip = strchr(token, 'c') != 0;
          
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
			}

			//Start Sound definitions
			if (strstr(token, ".wav") != NULL || strstr(token, ".mp3") != NULL || strstr(token, ".mid") != NULL)
			{
				hoverSound = token;
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
			}
			else if (strstr(token, ".none") != NULL)
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;

			if (strstr(token, ".wav") != NULL || strstr(token, ".mp3") != NULL || strstr(token, ".mid") != NULL)
			{
				pressedSound = token;
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
			}
			else if (strstr(token, ".none") != NULL)
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
			//End Sound definitions

			// ugly hack
			if (hasCommand)
			{
				char szCommands[MAX_LINE_LENGTH];
				char szLCommand[MAX_LINE_LENGTH] = { 0 };
				char szRCommand[MAX_LINE_LENGTH] = { 0 };
				char szMCommand[MAX_LINE_LENGTH] = { 0 };
				char szDropCommand[MAX_LINE_LENGTH] = { 0 };
            
				if (nextToken)
				{
					sprintf(szCommands, "[%s] %s", token, nextToken);
				}
				else
				{
					sprintf(szCommands, "[%s]", token);
				}

				if (token[0] == '[')
				{
					LPSTR lpszCommands[] = { szLCommand, szRCommand, szMCommand, szDropCommand };
					CommandTokenize(szCommands, lpszCommands, 4, NULL);
				}
				else
				{
					// compatibility mode, leftclick only
					strcpy(szLCommand, szCommands);
				}
           
				if (szLCommand[0])
				{
					fLCommand = new char[strlen(szLCommand) + 1];
					strcpy(fLCommand, szLCommand);
				}

				if (szRCommand[0])
				{
					fRCommand = new char[strlen(szRCommand) + 1];
					strcpy(fRCommand, szRCommand);
				}

				if (szMCommand[0])
				{
					fMCommand = new char[strlen(szMCommand) + 1];
					strcpy(fMCommand, szMCommand);
				}

				if (szDropCommand[0])
				{
					fDropCommand = new char[strlen(szDropCommand) + 1];
					strcpy(fDropCommand, szDropCommand);
				}
				else if (szLCommand[0])
				{
					// default drag&drop command to leftclick command
					fDropCommand = new char[strlen(szLCommand) + 1];
					strcpy(fDropCommand, szLCommand);
				}
			}
		}

		//----------------------------------------------------------------

		configSavePath = GetxRCString("Shortcut", "ConfigFile", "");	

		//Prepare "setBackground" Values
		//---------------------------------
		string images;
		images = normal;
		images.append(" ");
		images.append(hover);
		images.append(" ");
		images.append(pressed);

		if (Transparency)
			transparencyMode = 1;
		else
			transparencyMode = 3;
		//---------------------------------

		if (AcceptInput)
		{
			leftClickDownCommand = "";
			leftClickCommand = fLCommand;
			leftDoubleClickCommand = "";
			middleClickDownCommand = "";
			middleClickCommand = fMCommand;
			middleDoubleClickCommand = "";
			rightClickDownCommand = "";
			rightClickCommand = fRCommand;
			rightDoubleClickCommand = "";
			wheelDownCommand = "";
			wheelUpCommand = "";
			enterCommand = "";
			leaveCommand = "";
			dropCommand = "";
			textChangeCommand = "";
			resizeCommand = "";
			moveCommand = "";

			hideCommand = "";
			showCommand = "";
		}
	
		setGhosted(false);

		iconCommand = name;

		labelLeftClickRegions.clear();
		labelRightClickRegions.clear();
		labelMiddleClickRegions.clear();

		labelHREFClickRegions.clear();

		setScrolling(0);

		moveKey = 1;
		moveButton = 1;

		if ( !IsOS(OS_2KXP) || box != NULL )
			alphaMap = false;

		setBackground( GetShortcutTexture(images, (alphaMap ? "true" : "false") ) );	
		transparencyNeeded = background->isTransparencyNeeded();

		if (alphaMap)
			alphaMap = background->isAlphaMapNeeded();

		//setFont( GetRCFont("") );
		
		HBITMAP	checkbitmap = LoadLSImage(normal, 0);
		GetLSBitmapSize(checkbitmap, &actualwidth, &actualheight);
		if (checkbitmap)
			DeleteObject(checkbitmap);

		//Initial Repositioning
		if (box == NULL)
			reposition(xParseCoord(ix, 0, GetSystemMetrics(SM_CXSCREEN), lastx), xParseCoord(iy, 0, GetSystemMetrics(SM_CYSCREEN), lasty), actualwidth, actualheight);
		else
		{
			RECT rc;
			GetClientRect(box, &rc);
			reposition(xParseCoord(ix, 0, rc.right-rc.left, lastx),
						xParseCoord(iy, 0, rc.bottom-rc.top, lasty),
						actualwidth, actualheight);
		}

		if (hWnd)
		{
			SendMessage(hWnd, LM_SETCURSOR, 0, 0);

			if( alphaMap )
			{
				// MUST BE SET BEFORE setAlwaysOnTop()
				ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

				layeredFlag = true;

				UpdateLabelAlpha(255, true);
			}

			//File Actions
			DragAcceptFiles(hWnd, FALSE);

			//AlwaysOnTop
			setAlwaysOnTop(alwaysOnTop);

			//Initial Show
			IsVisible ? show() : hide();
		}

		//Update Region (Magic Pink)
		if (transparencyMode == 1)
			updateRegion();

		//CleanUp
		if (fLCommand)
			delete[] fLCommand;
		if (fRCommand)
			delete[] fRCommand;
		if (fMCommand)
			delete[] fMCommand;
		if (fDropCommand)
			delete[] fDropCommand;

		update();

		return UseTooltip;
	}

//----------------------------------------------------------------
#endif

//------------------------------------------------------------
//Set Background
//------------------------------------------------------------
void Label::setBackground(Texture *newbackground)
{
	if(!bUsingDefSkin)
		delete background;

	background = newbackground;

	bUsingDefSkin = (background == defaultSettings->skin);	
}

//------------------------------------------------------------
//Set Parent Box (nearly never needed/called!)
//------------------------------------------------------------
void Label::setBox(HWND newparent)
{
	if (box == newparent)
		return;

	ModifyStyleEx(hWnd, WS_EX_LAYERED, 0);

	if (alwaysOnTop)
		setAlwaysOnTop(FALSE);
	
	box = newparent;
	
	ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
	SetParent(hWnd, newparent);
}

//------------------------------------------------------------
//"Modify Styles"
//------------------------------------------------------------
void Label::setAlwaysOnTop(bool alwaysOnTop)
{
	if (box)
		return;
	
	this->alwaysOnTop = alwaysOnTop;
	
	if(hWnd)
	{
		ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
		SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
		ModifyStyle(hWnd, WS_CHILD, WS_POPUP);
		
		SetWindowPos(hWnd, alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
			0, 0, 0, 0,
			SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
	}
}

void Label::setAlpha(int newAlphaTransparency, bool updateValue)
{
	if (box)
		return;

	if (newAlphaTransparency >= 0 && newAlphaTransparency <= 255 && IsOS(OS_2KXP) )
	{
		if (!layeredFlag)
		{
			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, 0);
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			layeredFlag = true;
		}
		
		if (updateValue)
		{
			alphaTransparency = newAlphaTransparency;
		}
		
		UpdateLabelAlpha(newAlphaTransparency, true);
	}
}

void Label::setGhosted(bool ghosted)
{	
	if (box)
		return;

	this->ghosted = ghosted;
	
	if(hWnd)
	{
		if (ghosted)
			ModifyStyleEx(hWnd, 0, WS_EX_TRANSPARENT);
		else
			ModifyStyleEx(hWnd, WS_EX_TRANSPARENT, 0);
	}
}

//------------------------------------------------------------
//Set Font
//------------------------------------------------------------
void Label::setFont(xFont *newfont)
{
	if(font != defaultSettings->font)
		delete font;

	font = newfont;
}

//------------------------------------------------------------
//Set Text Settings
//------------------------------------------------------------
void Label::setText(const string &text, bool updateBackup)
{
	//Backup User Changed TEXT <-> Internal Change
	if ( updateBackup )
		originalTextBackup = text;

	//Clear previous <a href> lines and create new if text is painted!
	labelHREFClickRegions.clear();
	
	string process = text;
	if (!process.empty())
	{
		while ( strstr(process.c_str(), "\\n") != NULL )
		{
			process.replace(process.find("\\n"), 2, "<br>", 4);
		}
	}

	if (noTextSplit)
		originalText = split(process, "%&*&%");
	else
		originalText = split(process, ";");

	if ( updateBackup )
	{
		current = 0;
		currentBackup = 0;
	}
	else
		current = currentBackup;

	update();
}

void Label::previous()
{
	//Clear previous <a href> lines and create new if text is painted!
	labelHREFClickRegions.clear();

	current--;
	if(current < 0) current = originalText.size() - 1;
	currentBackup = current;

	update();
}

void Label::next()
{
	//Clear previous <a href> lines and create new if text is painted!
	labelHREFClickRegions.clear();

	current++;
	if(current >= originalText.size()) current = 0;
	currentBackup = current;

	update();
}

void Label::setScrolling(int scrolling)
{
	if (scrolling != 0 && scroll == 0)
	{
		scrollPosition = 0;
		SetTimer(hWnd, TIMER_SCROLL, scrollInterval, 0);
	}
	else if (scrolling == 0 && scroll != 0)
	{
		KillTimer(hWnd, TIMER_SCROLL);
		scrollPosition = 0;
		repaint();
	}

	scrollLimit = 0;
	scroll = scrolling;
	if (scrolling != 0)
		scrollBackup = scrolling;
}

void Label::setScrollLimit(int limit)
{
	if (limit >= 0)
	{
		setScrolling(scrollBackup);
		scrollLimit = limit + 1;
	}
	else
		setScrolling(0);
}

void Label::update()
{
	string oldText = text;

	//Tooltips
	if (!originalTooltip.empty() && TooltipHints != 0)
	{
		strcpy(tooltip, (systemInfo->processLabelText(originalTooltip, this, &dynamicTooltip)).c_str());	

		if (dynamicTooltip)
			updateHint(hWnd, tooltip);
	}

	//Process Escape Srquence for ';'
	string process = originalText[current];
	while ( strstr(process.c_str(), "%~") != NULL )
	{
		process.replace(process.find("%~"), 2, ";", 1);
	}

	if (!noEscape)
		text = systemInfo->processLabelText(process, this, &dynamicText);
	else
	{
		text = process;
		dynamicText = false;
	}

	if(hWnd)
	{
		if(dynamicText || dynamicTooltip || alwaysUpdateContent)
			SetTimer(hWnd, TIMER_UPDATE, updateInterval, 0);
		else
			KillTimer(hWnd, TIMER_UPDATE);
	}

	if (transparencyMode == 1 && transparencyNeeded && bufferDC)
		onPaint(bufferDC);
	else
		repaint();

	if (text != oldText)
	{
		if (autoWidthMode != 0 || autoHeightMode != 0)
			autoSize();
		if (transparencyMode == 1 && labelType != 1)
			updateRegion();
		if(!textChangeCommand.empty())
			xLSExecute(NULL, textChangeCommand.c_str(), SW_SHOWNORMAL);
	}
}

//------------------------------------------------------------
//Set Animations
//------------------------------------------------------------
void Label::setAnimation(bool start, int loops, int startFrame)
{
	imageLoop = loops;
	normalAnim = start;

	if (start)
	{
		frameCounter = startFrame;
		loopCounter = 0;
		SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
	}
	else
	{
		frameCounter = 0;
		loopCounter = 0;
		KillTimer(hWnd, TIMER_FRAMEANIMATION);
		repaint(true);
	}	
}

//------------------------------------------------------------
//Move/Resize/Reposition
//------------------------------------------------------------
void Label::move(int newx, int newy, int steps, int time, bool moveBy, bool command)
{
	POINT posHolder;
	posHolder.x = actualx;
	posHolder.y = actualy;

	if ( steps > 0 && time > 0 )
	{
		if (time > 50)
			time = 50;

		if (moveBy)
		{
			newx = actualx + newx;
			newy = actualy + newy;
		}

		int counter = 0;
		int deltax = newx - actualx;
		int deltay = newy - actualy;

		bool growx;
		bool growy;
		if (deltax < 0)
			growx = false;
		else
			growx = true;

		if (deltay < 0)
			growy = false;
		else
			growy = true;

		div_t div_result = div( deltax, steps );
		deltax = div_result.quot;
		div_result = div( deltay, steps );
		deltay = div_result.quot;

		if (deltax == 0 && growx)
			deltax = 1;
		else if (deltax == 0 && !growx)
			deltax = -1;

		if (deltay == 0 && growy)
			deltay = 1;
		else if (deltay == 0 && !growy)
			deltay = -1;

		int dynx = actualx;
		int dyny = actualy;

		if(hWnd)
		{
			int stopper = 0;
			while (counter <= steps)
			{
				stopper = 0; // Stop if x and y finished, stopper = 2
				dynx = dynx+deltax;
				if (growx && dynx > newx)
				{
					dynx = newx;
					stopper++;
				}
				else if (!growx && dynx < newx)
				{
					dynx = newx;
					stopper++;
				}

				dyny = dyny+deltay;
				if (growy && dyny > newy)
				{
					dyny = newy;
					stopper++;
				}
				else if (!growy && dyny < newy)
				{
					dyny = newy;
					stopper++;
				}
				
				SetWindowPos(hWnd, 0, dynx, dyny, 0, 0,
					SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER);

				if (box != NULL)
					SendMessage(box, WM_PAINT, 0, 0);

				SendMessage(hWnd, WM_PAINT, 0, 0);

				if (stopper == 2)
					counter = steps;
				
				counter++;

				Sleep(time);
			}

			actualx = newx;
			actualy = newy;

			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
					SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER);
		}
	}
	else
	{
		if(hWnd)
		{
			if (moveBy)
			{
				newx = actualx + newx;
				newy = actualy + newy;
			}

			actualx = newx;
			actualy = newy;

			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
				SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER);

			//Force Repaint
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}

	/*if (labelType == 0)
	{
		int temp = 0;
		string newpos = "";
		
		if (autoWidthMode == 0 || autoWidthMode == 1)
		{
			if ( actualx < 0 )
			{
				temp = actualx * (-1);
				newpos = ("~" + IntToString( temp ));
			}
			else
				newpos = IntToString( actualx );

			PrefixedLSSetVariable( name, "X", newpos );
		}

		if (autoHeightMode == 0 || autoHeightMode == 1)
		{
			if ( actualy < 0 )
			{
				temp = actualy * (-1);
				newpos = ("~" + IntToString( temp ));
			}
			else
				newpos = IntToString( actualy );

			PrefixedLSSetVariable( name, "Y", newpos );
		}
	}*/

	if (autoWidthMode != 0 || autoHeightMode != 0)
	{
		startCenterX = actualx+(actualwidth/2);
		startCenterY = actualy+(actualheight/2);
	}

	if(command && labelType == 0 && !moveCommand.empty() && (actualx != posHolder.x || actualy != posHolder.y))
		xLSExecute(hWnd, moveCommand.c_str(), SW_SHOWNORMAL);
}

void Label::resize(int newwidth, int newheight, int steps, int time, bool resizeBy)
{
	POINT sizeHolder;
	sizeHolder.x = actualwidth;
	sizeHolder.y = actualheight;
	if ( steps > 0 && time > 0 )
	{
		if (time > 50)
			time = 50;

		if (resizeBy)
		{
			newwidth = actualwidth + newwidth;
			newheight = actualheight + newheight;
		}

		int counter = 0;
		int deltax = newwidth - actualwidth;
		int deltay = newheight - actualheight;

		bool growx;
		bool growy;
		if (deltax < 0)
			growx = false;
		else
			growx = true;

		if (deltay < 0)
			growy = false;
		else
			growy = true;

		div_t div_result = div( deltax, steps );
		deltax = div_result.quot;
		div_result = div( deltay, steps );
		deltay = div_result.quot;

		if (deltax == 0 && growx)
			deltax = 1;
		else if (deltax == 0 && !growx)
			deltax = -1;
		if (deltay == 0 && growy)
			deltay = 1;
		else if (deltay == 0 && !growy)
			deltay = -1;

		int dynwidth = actualwidth;
		int dynheight = actualheight;

		if(hWnd)
		{
			int stopper = 0;
			while (counter <= steps)
			{
				stopper = 0;  // Stop if width and height are finished, stopper = 2
				dynwidth = dynwidth+deltax;
				if (growx && dynwidth > newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}
				else if (!growx && dynwidth < newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}

				dynheight = dynheight+deltay;
				if (growy && dynheight > newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				else if (!growy && dynheight < newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				
				SetWindowPos(hWnd, 0, 0, 0, dynwidth, dynheight,
					SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
				
				if (box != NULL)
					SendMessage(box, WM_PAINT, 0, 0);

				SendMessage(hWnd, WM_PAINT, 0, 0);

				if (transparencyMode == 1)
					updateRegion();

				if (stopper == 2)
					counter = steps;

				counter++;

				Sleep(time);
			}

			actualheight = newheight;
			actualwidth = newwidth;

			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
					SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
		
			//Force Repaint to get correct values for Resize Command (SetEvar must be called befire!)
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}
	else
	{
		if(hWnd)
		{
			if (resizeBy)
			{
				newwidth = actualwidth + newwidth;
				newheight = actualheight + newheight;
			}

			actualheight = newheight;
			actualwidth = newwidth;

			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
				SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);

			//Force Repaint to get correct values for Resize Command (SetEvar must be called before!)
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}

	if (transparencyMode == 1 && labelType != 1)
		updateRegion();

	/*if (labelType == 0)
	{
		if (autoWidthMode == 0)
			PrefixedLSSetVariable( name, "Width", IntToString(actualwidth) );
		if (autoHeightMode == 0)
			PrefixedLSSetVariable( name, "Height", IntToString(actualheight) );
	}*/

	if(!resizeCommand.empty() && (actualwidth != sizeHolder.x || actualheight != sizeHolder.y))
		xLSExecute(hWnd, resizeCommand.c_str(), SW_SHOWNORMAL);

	//Set new Rect for ToolTip Tracker
	if (TooltipHints)
		updateHint(hWnd, tooltip, true);
}

void Label::reposition(int newx, int newy, int newwidth, int newheight, int steps, int time, bool repositionBy)
{
	POINT posHolder;
	posHolder.x = actualx;
	posHolder.y = actualy;
	POINT sizeHolder;
	sizeHolder.x = actualwidth;
	sizeHolder.y = actualheight;
	if (steps > 0 && time > 0)
	{
		if (time > 50)
			time = 50;

		if (repositionBy)
		{
			newx = actualx + newx;
			newy = actualy + newy;
			newwidth = actualwidth + newwidth;
			newheight = actualheight + newheight;
		}

		int counter = 0;
		int deltax = newx - actualx;
		int deltay = newy - actualy;
		int deltasizex = newwidth - actualwidth;
		int deltasizey = newheight - actualheight;

		bool growx;
		bool growy;
		bool growsizex;
		bool growsizey;
		
		if (deltax < 0)
			growx = false;
		else
			growx = true;
		if (deltay < 0)
			growy = false;
		else
			growy = true;
		
		if (deltasizex < 0)
			growsizex = false;
		else
			growsizex = true;
		if (deltasizey < 0)
			growsizey = false;
		else
			growsizey = true;

		div_t div_result;
		div_result = div( deltax, steps );
		deltax = div_result.quot;
		div_result = div( deltay, steps );
		deltay = div_result.quot;

		div_result = div( deltasizex, steps );
		deltasizex = div_result.quot;
		div_result = div( deltasizey, steps );
		deltasizey = div_result.quot;

		if (deltax == 0 && growx)
			deltax = 1;
		else if (deltax == 0 && !growx)
			deltax = -1;
		if (deltay == 0 && growy)
			deltay = 1;
		else if (deltay == 0 && !growy)
			deltay = -1;

		if (deltasizex == 0 && growsizex)
			deltasizex = 1;
		else if (deltasizex == 0 && !growsizex)
			deltasizex = -1;
		if (deltasizey == 0 && growsizey)
			deltasizey = 1;
		else if (deltasizey == 0 && !growsizey)
			deltasizey = -1;

		int dynx = actualx;
		int dyny = actualy;
		int dynwidth = actualwidth;
		int dynheight = actualheight;

		if(hWnd)
		{
			int stopper = 0;
			while (counter <= steps)
			{
				stopper = 0; // Stop if x, y, width and height are finished, stopper = 4
				dynx = dynx+deltax;
				if (growx && dynx > newx)
				{
					dynx = newx;
					stopper++;
				}
				else if (!growx && dynx < newx)
				{
					dynx = newx;
					stopper++;
				}

				dyny = dyny+deltay;
				if (growy && dyny > newy)
				{
					dyny = newy;
					stopper++;
				}
				else if (!growy && dyny < newy)
				{
					dyny = newy;
					stopper++;
				}

				dynwidth = dynwidth+deltasizex;
				if (growsizex && dynwidth > newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}
				else if (!growsizex && dynwidth < newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}

				dynheight = dynheight+deltasizey;
				if (growsizey && dynheight > newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				else if (!growsizey && dynheight < newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				
				SetWindowPos(hWnd, 0, dynx, dyny, dynwidth, dynheight,
					SWP_NOACTIVATE | SWP_NOZORDER);
				
				if (box != NULL)
					SendMessage(box, WM_PAINT, 0, 0);

				SendMessage(hWnd, WM_PAINT, 0, 0);

				if (transparencyMode == 1)
					updateRegion();

				if (stopper == 4)
					counter = steps;

				counter++;

				Sleep(time);
			}

			actualx = newx;
			actualy = newy;
			actualwidth = newwidth;
			actualheight = newheight;

			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
				SWP_NOACTIVATE | SWP_NOZORDER);

			//Force Repaint to get correct values for Resize Command (SetEvar must be called befire!)
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}
	else
	{
		if(hWnd)
		{
			if (repositionBy)
			{
				newx = actualx + newx;
				newy = actualy + newy;
				newwidth = actualwidth + newwidth;
				newheight = actualheight + newheight;
			}

			actualx = newx;
			actualy = newy;
			actualheight = newheight;
			actualwidth = newwidth;

			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
				SWP_NOACTIVATE | SWP_NOZORDER);

			//Force Repaint to get correct values for Resize Command (SetEvar must be called befire!)
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}

	if (transparencyMode == 1 && labelType != 1)
		updateRegion();

	/*if (labelType == 0)
	{
		int temp = 0;
		string newpos = "";
		
		if (autoWidthMode == 0 || autoWidthMode == 1)
		{
			if ( actualx < 0 )
			{
				temp = actualx * (-1);
				newpos = ("~" + IntToString( temp ));
			}
			else
				newpos = IntToString( actualx );

			PrefixedLSSetVariable( name, "X", newpos );
		}

		if (autoHeightMode == 0 || autoHeightMode == 1)
		{
			if ( actualy < 0 )
			{
				temp = actualy * (-1);
				newpos = ("~" + IntToString( temp ));
			}
			else
				newpos = IntToString( actualy );

			PrefixedLSSetVariable( name, "Y", newpos );
		}

		if (autoWidthMode == 0)
			PrefixedLSSetVariable( name, "Width", IntToString(actualwidth) );
		if (autoHeightMode == 0)
			PrefixedLSSetVariable( name, "Height", IntToString(actualheight) );
	}*/

	if (autoWidthMode != 0 || autoHeightMode != 0)
	{
		startCenterX = actualx+(actualwidth/2);
		startCenterY = actualy+(actualheight/2);
	}

	if(!resizeCommand.empty() && (actualwidth != sizeHolder.x || actualheight != sizeHolder.y))
		xLSExecute(hWnd, resizeCommand.c_str(), SW_SHOWNORMAL);

	if(labelType == 0 && !moveCommand.empty() && (actualx != posHolder.x || actualy != posHolder.y))
		xLSExecute(hWnd, moveCommand.c_str(), SW_SHOWNORMAL);

	//Set new Rect for ToolTip Tracker
	if (TooltipHints)
		updateHint(hWnd, tooltip, true);
}

//------------------------------------------------------------
//Show/Hide
//------------------------------------------------------------
void Label::show(bool execute)
{
	if (!hWnd && labelType == 0)
		load(hInstance, box);
	
	if(hWnd)
	{
		KillTimer(hWnd, TIMER_AUTOHIDE);

		if (alphaFade && !visible)
		{
			ShowWindow(hWnd, SW_SHOWNOACTIVATE);

			KillTimer(hWnd, TIMER_ALPHAFADEOUT);
			SetTimer(hWnd, TIMER_ALPHAFADEIN, alphaFadeDelay, NULL);
		}
		else
			ShowWindow(hWnd, SW_SHOWNOACTIVATE);

		if(!showCommand.empty() && execute && !visible)
			xLSExecute(hWnd, showCommand.c_str(), SW_SHOWNORMAL);

		visible = true;

		if (normalAnim)
		{
			frameCounter = 1;
			loopCounter = 0;
			SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
		}
	}
}

void Label::showHide(double timeout)
{
	show();
	SetTimer(hWnd, TIMER_AUTOHIDE, timeout * 1000, NULL);
}

void Label::hide(bool execute)
{
	if(hWnd)
	{
		if (alphaFade && visible)
		{
			UpdateAlpha = alphaTransparency;

			KillTimer(hWnd, TIMER_ALPHAFADEIN);
			SetTimer(hWnd, TIMER_ALPHAFADEOUT, alphaFadeDelay, NULL);
		}
		else
			ShowWindow(hWnd, SW_HIDE);

		if(!hideCommand.empty() && execute && visible)
			xLSExecute(hWnd, hideCommand.c_str(), SW_SHOWNORMAL);

		visible = false;

		KillTimer(hWnd, TIMER_AUTOHIDE);

		if (normalAnim || hoverAnim)
			KillTimer(hWnd, TIMER_FRAMEANIMATION);
	}
}


//------------------------------------------------------------
//Clipboard Functions
//------------------------------------------------------------
void Label::clipboardCopy(string prefix)
{
	string toCopy = prefix.empty() ? text : (prefix + " " + text);
	SetClipboardText(hWnd, toCopy.c_str());
}

void Label::clipboardPaste(string prefix)
{
	char buffer[256];

	if(GetClipboardText(hWnd, buffer, 256))
	{
		string toPaste = prefix.empty() ? string(buffer) : (prefix + " " + string(buffer));
		setText(toPaste);
	}
}

//------------------------------------------------------------
//Region Events (Checking)
//------------------------------------------------------------
bool Label::checkEnterLeaveRegion(int x, int y, StringList inputlist)
{
	bool found = false;
	if (!inputlist.empty())
	{
		for(StringListIterator it = inputlist.begin(); it != inputlist.end(); it++)
		{
			char left[16], top[16], right[16], bottom[16], command[MAX_PATH_LENGTH];
			char *buffers[] = {left, top, right, bottom};

			if(LCTokenize((*it).c_str(), buffers, 4, command) >= 4)
			{
				int intleft = xParseCoord(left, 0, actualwidth);
				int intright = xParseCoord(right, -0, actualwidth);
				int inttop = xParseCoord(top, 0, actualheight);
				int intbottom  = xParseCoord(bottom, -0, actualheight);

				if (intleft < intright && inttop < intbottom)
				{
					if(x >= intleft && x <= intright && y >= inttop && y <= intbottom)
					{
						if (!handcursor)
						{
							// Get the windows directory
							CString strWndDir;
							GetWindowsDirectory(strWndDir.GetBuffer(MAX_PATH_LENGTH), MAX_PATH_LENGTH);
							strWndDir.ReleaseBuffer();

							strWndDir += _T("\\winhlp32.exe");

							// This retrieves cursor #106 from winhlp32.exe, which is a hand pointer
							HMODULE hModule = LoadLibrary(strWndDir);
							
							if (hModule)
								handcursor = ::LoadCursor(hModule, MAKEINTRESOURCE(106));
				
							FreeLibrary(hModule);
						}

						if (handcursor)
							SetCursor(handcursor);
						
						//Processing Hover Change
						if (hrefconfig.size() == 2 && !mouseInsideHREF)
						{
							CString textprocess = originalText[current].c_str();
							CString textoutput = originalText[current].c_str();
							CString commandprocess = command;
							commandprocess += "'>";

							textprocess.MakeLower();
							commandprocess.MakeLower();

							int pos = 0;
							pos = textprocess.Find(commandprocess);		
							pos = textprocess.Find("'>", pos);
							textprocess.Insert(pos+2, hrefconfig[0].c_str());
							textoutput.Insert(pos+2, hrefconfig[0].c_str());
							pos = textprocess.Find("</a>", pos+2);
							textprocess.Insert(pos, hrefconfig[1].c_str());
							textoutput.Insert(pos, hrefconfig[1].c_str());

							textoutput.Replace("\"", "");

							setText((string)textoutput);
						}
						//End

						mouseInsideHREF = true;
						found = true;
						break;
					}
				}
			}
		}
	}

	return found;
}

bool Label::checkRegion(int x, int y, StringList inputlist)
{
	bool found = false;
	if (!inputlist.empty())
	{
		for(StringListIterator it = inputlist.begin(); it != inputlist.end(); it++)
		{
			char left[16], top[16], right[16], bottom[16], command[MAX_PATH_LENGTH];
			char *buffers[] = {left, top, right, bottom};

			if(LCTokenize((*it).c_str(), buffers, 4, command) >= 4)
			{
				int intleft = xParseCoord(left, 0, actualwidth);
				int intright = xParseCoord(right, -0, actualwidth);
				int inttop = xParseCoord(top, 0, actualheight);
				int intbottom  = xParseCoord(bottom, -0, actualheight);

				if (intleft < intright && inttop < intbottom)
				{
					if(x >= intleft && x <= intright && y >= inttop && y <= intbottom)
					{	
						string process = command;
						process = lowerCase(process);
						if (strstr(process.c_str(), "http:") != NULL)
							GotoURL(command, SW_SHOWNORMAL);
						else
						{
							string bang = command;
							LSExecute(hWnd, bang.c_str(), SW_SHOWNORMAL);
						}

						found = true;
						break;	
					}
				}
			}
		}
	}
	return found;
}

//------------------------------------------------------------
//Mouse Related Actions (Clicks, Enter, Leave, Move, ...)
//------------------------------------------------------------

void Label::onLButtonDblClk(int x, int y)
{
	if(!leftDoubleClickCommand.empty())
		xLSExecute(hWnd, leftDoubleClickCommand.c_str(), SW_SHOWNORMAL);
}

void Label::onLButtonDown(int x, int y)
{
	// 1 = LeftButton
	ontemplateButtonDown(1, leftClickDownCommand);
}

void Label::onLButtonUp(int x, int y)
{	
	// 1 = LeftButton
	ontemplateButtonUp(x, y, 1,labelLeftClickRegions, leftClickCommand);
}

void Label::onMButtonDblClk(int x, int y)
{
	if(!middleDoubleClickCommand.empty())
		xLSExecute(hWnd, middleDoubleClickCommand.c_str(), SW_SHOWNORMAL);
}

void Label::onMButtonDown(int x, int y)
{
	// 3 = MiddleButton
	ontemplateButtonDown(3, middleClickDownCommand);
}

void Label::onMButtonUp(int x, int y)
{
	// 3 = MiddleButton
	ontemplateButtonUp(x, y, 3, labelMiddleClickRegions, middleClickCommand);
}

void Label::onRButtonDblClk(int x, int y)
{
	if(!rightDoubleClickCommand.empty())
		xLSExecute(hWnd, rightDoubleClickCommand.c_str(), SW_SHOWNORMAL);
}

void Label::onRButtonDown(int x, int y)
{
	// 2 = RightButton
	ontemplateButtonDown(2, rightClickDownCommand);
}

void Label::onRButtonUp(int x, int y)
{
	// 2 = RightButton
	ontemplateButtonUp(x, y, 2, labelRightClickRegions, rightClickCommand);
}

void Label::onWheelDown(int x, int y)
{
	if(!wheelDownCommand.empty())
		xLSExecute(hWnd, wheelDownCommand.c_str(), SW_SHOWNORMAL);

	if (focusOnEnter == 1)
	{
		focusOnEnter = 2;
		mouse_event(MOUSEEVENTF_MIDDLEDOWN, NULL, NULL, NULL, NULL);
		mouse_event(MOUSEEVENTF_MIDDLEUP, NULL, NULL, NULL, NULL);
	}
}

void Label::onWheelUp(int x, int y)
{
	if(!wheelUpCommand.empty())
		xLSExecute(hWnd, wheelUpCommand.c_str(), SW_SHOWNORMAL);

	if (focusOnEnter == 1)
	{
		focusOnEnter = 2;
		mouse_event(MOUSEEVENTF_MIDDLEDOWN, NULL, NULL, NULL, NULL);
		mouse_event(MOUSEEVENTF_MIDDLEUP, NULL, NULL, NULL, NULL);
	}
}

void Label::onMouseEnter()
{
	SendMessage(hWnd, LM_SETCURSOR, 0, 0);

	if (useHover)
	{
		//Kill AnimTimer from Normal Animation
		if (normalAnim)
			KillTimer(hWnd, TIMER_FRAMEANIMATION);

		//If HoverAnim is set (hoverframecount != 0)
		if (hoverAnim)
		{
			hoverActive = true;
			pressedActive = false;
			frameCounter = 1;
			loopCounter = 0;
			//Start AnimTimer from Hover Animation
			SetTimer(hWnd, TIMER_FRAMEANIMATION, hoverimageFrameDelay, NULL);
		}
		//If HoverAnim is not set (hoverframecount == 0)
		else
		{
			frameCounter = 0;
			hoverActive = true;
			pressedActive = false;
			repaint(true);

			if (transparencyMode == 1 && labelType != 1)
				updateRegion();
		}
	}

	#ifndef xLabelLight
	//----------------------------------------------------------------
		if (labelType == 2 && !hoverSound.empty())
			ParseBangCommand(hWnd, "!playsound", hoverSound.c_str());
	//----------------------------------------------------------------
	#endif
	
	if(!enterCommand.empty())
		xLSExecute(hWnd, enterCommand.c_str(), SW_SHOWNORMAL);

	#ifndef xLabelLight
	//----------------------------------------------------------------
		if ( labelType == 1 && easyDesktopFocus )
			SendMessage(hWnd, WM_ACTIVATE, WA_CLICKACTIVE, 0);
	//----------------------------------------------------------------
	#endif
		
	if (focusOnEnter == 1)
	{
		focusOnEnter = 2;
		mouse_event(MOUSEEVENTF_MIDDLEDOWN, NULL, NULL, NULL, NULL);
		mouse_event(MOUSEEVENTF_MIDDLEUP, NULL, NULL, NULL, NULL);
	}
}

void Label::onMouseLeave()
{
	if (useHover || usePressed)
	{
		//If HoverAnim was active (hoverframecount != 0)
		if (hoverAnim)
		{
			hoverActive = false;
			pressedActive = false;
			frameCounter = 0;
			//Kill AnimTimer from Hover Animation
			KillTimer(hWnd, TIMER_FRAMEANIMATION);

			//If NormalAnim is set (framecount != 0)
			if (normalAnim)
			{
				frameCounter = 1;
				loopCounter = 0;
				SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
			}
			//If NormalAnim is not set (framecount == 0)
			else
			{
				if ( labelType != 1 )
				{
					repaint(true);
					if (transparencyMode == 1)
						updateRegion();
				}
			}
		}
		else
		{
			hoverActive = false;
			pressedActive = false;
			//If NormalAnim is set (framecount != 0)
			if (normalAnim)
			{
				frameCounter = 1;
				SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
			}
			//If NormalAnim is not set (framecount == 0)
			else
			{
				if ( labelType != 1 )
				{
					repaint(true);
					if (transparencyMode == 1)
						updateRegion();
				}
			}
		}
	}

	if(!leaveCommand.empty())
		xLSExecute(hWnd, leaveCommand.c_str(), SW_SHOWNORMAL);
	
	#ifndef xLabelLight
	//----------------------------------------------------------------
		if (labelType == 1 && easyDesktopFocus)
			SendMessage(hWnd, WM_ACTIVATE, WA_INACTIVE, 0);
	#endif xLabelLight
	//----------------------------------------------------------------
}

void Label::onMouseMove(int x, int y)
{
	if(moveable)
	{
		if (mousePressed && movemodifierkeyPressed && moveButtonPressed == moveButton)
		{
			KillTimer(hWnd, TIMER_MOUSETRACK);
			SetTimer(hWnd, TIMER_MOUSETRACK_MOVE, 1, 0);
		}
		else
		{
			KillTimer(hWnd, TIMER_MOUSETRACK_MOVE);
			SetTimer(hWnd, TIMER_MOUSETRACK, 100, 0);

			if(!mouseInside)
			{
				mouseInside = true;
				onMouseEnter();
				SetTimer(hWnd, TIMER_MOUSETRACK, 100, 0);
			}
		}
	}
	else if (!moveable)
	{
		if(!mouseInside)
		{
			mouseInside = true;
			onMouseEnter();
			SetTimer(hWnd, TIMER_MOUSETRACK, 100, 0);
		}
	}
}

//------------------------------------------------------------
//AutoWidthMode and AutoHeightMode Implementation
//------------------------------------------------------------
void Label::autoSize()
{
	RECT mainRect;
	GetWindowRect(hWnd, &mainRect);
	POINT boxConverted;

	boxConverted.x = mainRect.left;
	boxConverted.y = mainRect.top;
	if (box != NULL)
		ScreenToClient(box, &boxConverted);

	int width = mainRect.right - mainRect.left;
	int height = mainRect.bottom - mainRect.top;

	int textWidth;
	int textHeight;

	POINT size = font->measure(bufferDC, leftBorder, width - rightBorder, text, align, vertAlign);

	textWidth = size.x+6;
	textHeight = size.y+6;

	//For AUTOWIDTH/HEIGHT MODE	
	int newx;
	int newy;
	int newwidth;
	int newheight;

	int deltax;
	int deltay;

	if (autoWidthMode != 0)
		deltax = textWidth - (width - leftBorder - rightBorder);
	else
		deltax = 0;

	if (autoHeightMode != 0)
		deltay = textHeight - (height - topBorder - bottomBorder);
	else
		deltay = 0;

	newx = boxConverted.x;
	newy = boxConverted.y;
	newwidth = width + deltax;
	newheight = height + deltay;

	//Check MIN/MAX
	if (newwidth < autoMinWidth || newwidth > autoMaxWidth)
	{
		if (newwidth < autoMinWidth)
		{
			deltax = deltax + (autoMinWidth-newwidth);
			newwidth = autoMinWidth;
		}
		if (newwidth > autoMaxWidth)
		{
			deltax = deltax - (newwidth-autoMaxWidth);
			newwidth = autoMaxWidth;
		}
	}
	if (newheight < autoMinHeight || newheight > autoMaxHeight)
	{
		if (newheight < autoMinHeight)
		{
			deltay = deltay + (autoMinHeight-newheight);
			newheight = autoMinHeight;
		}
		if (newheight > autoMaxHeight)
		{
			deltay = deltay - (newheight-autoMaxHeight);
			newheight = autoMaxHeight;
		}
	}

	if ((autoWidthMode == 0 || autoWidthMode == 1) && (autoHeightMode == 0 || autoHeightMode == 1))
	{
		newx = boxConverted.x;
		newy = boxConverted.y;
	}
	else if ((autoWidthMode == 0 || autoWidthMode == 1) && autoHeightMode == 2)
	{
		newx = boxConverted.x;
		newy = boxConverted.y-deltay;
	}
	else if ((autoWidthMode == 0 || autoWidthMode == 1) && autoHeightMode == 3)
	{
		newx = boxConverted.x;
		newy = startCenterY-(newheight/2);
	}
	else if (autoWidthMode == 2 && (autoHeightMode == 0 || autoHeightMode == 1))
	{
		newx = boxConverted.x-deltax;
		newy = boxConverted.y;
	}
	else if (autoWidthMode == 2 && autoHeightMode == 2)
	{
		newx = boxConverted.x-deltax;
		newy = boxConverted.y-deltay;
	}
	else if (autoWidthMode == 2 && autoHeightMode == 3)
	{
		newx = boxConverted.x-deltax;
		newy = startCenterY-(newheight/2);
	}
	else if (autoWidthMode == 3 && (autoHeightMode == 0 || autoHeightMode == 1))
	{
		newx = startCenterX-(newwidth/2);
		newy = boxConverted.y;
	}
	else if (autoWidthMode == 3 && autoHeightMode == 2)
	{
		newx = startCenterX-(newwidth/2);
		newy = boxConverted.y-deltay;
	}
	else if (autoWidthMode == 3 && autoHeightMode == 3)
	{
		newx = startCenterX-(newwidth/2);
		newy = startCenterY-(newheight/2);
	}

	//Reposition to NEW Pos/Size ( ONLY IN AUTOWIDTH-, AUTOHEIGHT-MODE )
	reposition(newx, newy, newwidth, newheight);
}

void Label::repaint(bool invalidateCache)
{
	if(hWnd && !alphaMap)
	{
		if(invalidateCache)
		{
			if(backgroundDC && backgroundBitmap)
			{
				backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
				DeleteObject(backgroundBitmap);
				backgroundBitmap = 0;
			}

			if(bufferDC && bufferBitmap)
			{
				bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
				DeleteObject(bufferBitmap);
				bufferBitmap = 0;
			}
		}

		InvalidateRect(hWnd, NULL, false);
	}

	//Attention!! Background MUST exist
	if ( alphaMap && IsOS(OS_2KXP) && box == NULL && background )
		UpdateLabelAlpha(alphaTransparency, true);
}

void Label::onPaint(HDC hDC)
{
	int actualImageMode = 0;
	bool useFrames = false;
	RECT mainRect;
	GetWindowRect(hWnd, &mainRect);

	int width = mainRect.right - mainRect.left;
	int height = mainRect.bottom - mainRect.top;

	RECT completeFillRect;
	completeFillRect.left = 0;
	completeFillRect.top = 0;
	completeFillRect.right = width;
	completeFillRect.bottom = height;

	// keep a cached rendition of the background
	if(!backgroundBitmap)
	{
		if(!backgroundDC)
			backgroundDC = CreateCompatibleDC(hDC);

		backgroundBitmap = CreateCompatibleBitmap(hDC, width, height);
		SelectObject(backgroundDC, backgroundBitmap);

		//True Transparency
		if (transparencyMode == 1)
		{
			::SetBkColor(backgroundDC, RGB(255, 0, 255));
			::ExtTextOut(backgroundDC, 0, 0, ETO_OPAQUE, &completeFillRect, NULL, 0, NULL);
		}
		//PrePaint Background (Fake Transparency)
		else if(transparencyMode == 2)
		{
			if(box)
			{
				//  =======================================================================
				//  Copy boxbg to backgroundDC
				//  =======================================================================	
				char windowClass[MAX_PATH_LENGTH];
				strcpy(windowClass, "");

				GetClassName( box, windowClass, MAX_PATH_LENGTH );
				if (stricmp(windowClass, "xlabel") == NULL || stricmp(windowClass, "xlabelxp") == NULL)
				{
					LPSTR windowCaption = NULL;
		
					int length = GetWindowTextLength( box );

					windowCaption = new char[length + 1];
					GetWindowText( box, windowCaption, length + 1 );
					
					for(LabelListIterator lit = labelList.begin(); lit != labelList.end(); lit++)
					{
						if(stricmp(windowCaption, (*lit)->getName().c_str()) == NULL)
						{
							(*lit)->getBackground()->apply(backgroundDC, (*lit)->getX(), (*lit)->getY(), (*lit)->getWidth(), (*lit)->getHeight(), 1, 0);
							break;
						}
					}

					if (windowCaption)
						delete[] windowCaption;
				}
				else
				{
					BitBlt(backgroundDC, 0, 0, width, height, hDC, 0, 0, SRCCOPY);
				}
			}
			else
			{
				// paint desktop on display DC and then into background buffer
				PaintDesktopEx(backgroundDC, 0, 0, width, height, mainRect.left, mainRect.top, FALSE);
			}
		}
		//Default Background
		else if (transparencyMode == 3)
		{
			if (borderMethod == 2)
				DrawEdge(backgroundDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT | BF_MIDDLE);
			else if (borderMethod == 3)
				DrawEdge(backgroundDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT | BF_MIDDLE);
			else if (borderMethod == 4)
				DrawEdge(backgroundDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT | BF_MIDDLE);
			else
				DrawEdge(backgroundDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT | BF_MIDDLE);
		}
		
		//Normal Background Painting if no Animation
		if ( (!normalAnim && !hoverAnim) || frameCounter == 0 || (usePressed && pressedActive))
		{
			if (hoverActive && useHover)
			{
				background->apply(backgroundDC, 0, 0, width, height, transparencyMode, 1);
				actualImageMode = 1;
			}
			else if (pressedActive && usePressed)
			{
				background->apply(backgroundDC, 0, 0, width, height, transparencyMode, 2);
				actualImageMode = 2;
			}
			else
			{
				background->apply(backgroundDC, 0, 0, width, height, transparencyMode, 0);
				actualImageMode = 0;
			}
		}
		//Special Background Painting if Animation (normal or hover)
		else
		{
			useFrames = true;
			if (hoverActive && useHover)
				background->applyAnim(backgroundDC, 0, 0, width, height, transparencyMode, 1, frameCounter);
			else if (pressedActive && usePressed)
				background->applyAnim(backgroundDC, 0, 0, width, height, transparencyMode, 1, frameCounter);
			else
				background->applyAnim(backgroundDC, 0, 0, width, height, transparencyMode, 0, frameCounter);
		}
	}

	// double buffer for flicker-free paint
	if(!bufferBitmap)
	{
		if(!bufferDC)
			bufferDC = CreateCompatibleDC(hDC);

		bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		SelectObject(bufferDC, bufferBitmap);
	}

	// blt background into double buffer
	BitBlt(bufferDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);

	//Text Painting Function
	if ( !text.empty() )
		onTextPaint(bufferDC, width, height);

	//Border Painting
	if (hoverActive && useHover && hoverborderMethod != 0)
	{
		if (hoverborderMethod == 1)
			DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
		else if (hoverborderMethod == 2)
			DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
		else if (hoverborderMethod == 3)
			DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
		else if (hoverborderMethod == 4)
			DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	}
	else if (pressedActive && usePressed && pressedborderMethod != 0)
	{
		if (pressedborderMethod == 1)
			DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
		else if (pressedborderMethod == 2)
			DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
		else if (pressedborderMethod == 3)
			DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
		else if (pressedborderMethod == 4)
			DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	}
	else if (borderMethod != 0)
	{
		if (borderMethod == 1)
			DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
		else if (borderMethod == 2)
			DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
		else if (borderMethod == 3)
			DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
		else if (borderMethod == 4)
			DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	}

	//Normal Image Switch
	if (overblendStyle == 0 || useFrames || lastImageMode == actualImageMode || (transparencyMode == 1 && transparencyNeeded))
	{
		// blt the double buffer to the display
		BitBlt(hDC, 0, 0, width, height, bufferDC, 0, 0, SRCCOPY);
	}
	//OverblendStyle Image Switch
	else if (overblendStyle == -1)
		MagicShow( bufferDC, 0, 0, hDC, 0, 0, width, height, 5, 2, rand()%20 );
	else
		MagicShow( bufferDC, 0, 0, hDC, 0, 0, width, height, 5, 2, overblendStyle );

	lastImageMode = actualImageMode;
}

void Label::onTextPaint(HDC desthdc, int width, int height)
{
	//--------------------------------------------------------------------------------
	// start rendering text
	//--------------------------------------------------------------------------------
	//Create Region in which text drawing is allowed
	HRGN drawArea = CreateRectRgn( 0, 0, actualwidth, actualheight );
	HRGN allowedArea = CreateRectRgn( leftBorder, topBorder, width - rightBorder, height - bottomBorder );
	SelectClipRgn(desthdc, allowedArea);
	DeleteObject(allowedArea);
	
	int textWidth;
	int textHeight;

	POINT size = font->measure(desthdc, leftBorder, width - rightBorder, text, align, vertAlign);

	textWidth = size.x+6;
	textHeight = size.y+6;

	if ( ((scroll == 1 || scroll == 3) && textWidth > width - leftBorder - rightBorder) || ((scroll == 2 || scroll == 4) && textHeight > height - topBorder - bottomBorder) )
	{			
		if (scroll == 1 || scroll == 3)
		{
			if(scrollPosition <= 0)
			{
				scrollPosition = textWidth + scrollPadLength;
					
				if (scrollLimit)
				{
					scrollLimit--;
					if (scrollLimit == 0)
						setScrolling(0);
				}
			}

			if(scrollPosition > textWidth + scrollPadLength)
				scrollPosition = 0;
		}
		else if (scroll == 2 || scroll == 4)
		{
			if(scrollPosition <= 0)
			{
				scrollPosition = textHeight + scrollPadLength;
					
				if (scrollLimit)
				{
					scrollLimit--;
					if (scrollLimit == 0)
						setScrolling(0);
				}
			}

			if(scrollPosition > textHeight + scrollPadLength )
				scrollPosition = 0;
		}
		
		if (scroll == 1 || scroll == 3)
		{
			if (align == DT_CENTER)
			{
				font->apply(desthdc,
						scrollPosition,
						scrollPosition - scrollPadLength,
						topBorder,
						height - bottomBorder,
						text,
						0,
						vertAlign,
						labelHREFClickRegions);

				font->apply(desthdc,
						scrollPosition - scrollPadLength - textWidth,
						scrollPosition + textWidth,
						topBorder,
						height - bottomBorder,
						text,
						0,
						vertAlign,
						labelHREFClickRegions);
			}
			else
			{
				font->apply(desthdc,
						scrollPosition,
						scrollPosition - scrollPadLength,
						topBorder,
						height - bottomBorder,
						text,
						align,
						vertAlign,
						labelHREFClickRegions);

				font->apply(desthdc,
						scrollPosition - scrollPadLength - textWidth,
						scrollPosition + textWidth,
						topBorder,
						height - bottomBorder,
						text,
						align,
						vertAlign,
						labelHREFClickRegions);
			}
		}
		else if (scroll == 2 || scroll == 4)
		{
			font->apply(desthdc,
					leftBorder,
					width - rightBorder,
					scrollPosition,
					scrollPosition - scrollPadLength,
					text,
					align,
					2,
					labelHREFClickRegions);

			font->apply(desthdc,
					leftBorder,
					width - rightBorder,
					scrollPosition - scrollPadLength - textHeight,
					scrollPosition + textHeight,
					text,
					align,
					2,
					labelHREFClickRegions);
		}
	}
	else
	{
		font->apply(desthdc,
					leftBorder,
					width - rightBorder,
					topBorder,
					height - bottomBorder,
					text,
					align,
					vertAlign,
					labelHREFClickRegions,
					(labelType == 1 ? true : false));
	}

	//Reset Region in which text drawing is allowed
	SelectClipRgn(desthdc, drawArea);
	DeleteObject(drawArea);
	//--------------------------------------------------------------------------------
	// stop rendering text
	//--------------------------------------------------------------------------------
}

void Label::updateRegion()
{
	if ( transparencyNeeded && !alphaMap )
	{
		HDC tmpDC = CreateCompatibleDC(0);
		HBITMAP tmpBmp = CreateBitmap(actualwidth, actualheight, 1, 32, NULL);
		HGDIOBJ tmpObj = SelectObject(tmpDC, tmpBmp);

		//Draw Image
		if (bufferDC && bufferBitmap)
			BitBlt(tmpDC, 0, 0, actualwidth, actualheight, bufferDC, 0, 0, SRCCOPY);
		else
			onPaint(tmpDC);
	
		SelectObject(tmpDC, tmpObj);

		HRGN region = BitmapToRegion(tmpBmp, RGB(255, 0, 255), 0, 0, 0);
		HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(windowRgn, region, NULL, RGN_COPY);

		SetWindowRgn(hWnd, windowRgn, true);	

		DeleteObject(windowRgn);
		DeleteObject(region);
		DeleteObject(tmpBmp);
		DeleteObject(tmpObj);
		DeleteDC(tmpDC);
	}
	else if ( !alphaMap )
	{
		HRGN windowRgn = CreateRectRgn(0, 0, actualwidth, actualheight);
		SetWindowRgn(hWnd, windowRgn, false);
		DeleteObject(windowRgn);
	}
}

void Label::onWindowPosChanged(WINDOWPOS *windowPos)
{
	repaint(true);
	
	RedrawWindow(hWnd, NULL, NULL, RDW_UPDATENOW);

	if (labelType == 0 || labelType == 1)
	{
		//Check Snap to Borders
		if (mousePressed && movemodifierkeyPressed && (snapDistance != 0 || (labelType == 1 && DISnapTo)))
		{
			POINT newpt;
			GetCursorPos(&newpt);
			
			POINT pos = {windowPos->x, windowPos->y};
			POINT size = {windowPos->cx, windowPos->cy};

			int result = 0;

			//Check First WorkArea Border
			if ( (stricmp("alllabels", snapTo.c_str()) == NULL || stricmp("desktop", snapTo.c_str()) == NULL) && !DISnapTo )
			{
				RECT rScreen;
				SystemParametersInfo(SPI_GETWORKAREA, 0, &rScreen, NULL);
				result = isCloseTo( pos, size, rScreen.left, rScreen.top, rScreen.right-rScreen.left, rScreen.bottom-rScreen.top, snapDistance);
			}

			if (result == 0)
			{
				if (labelType == 0)
				{
					for( LabelListIterator lit = labelList.begin(); lit != labelList.end(); lit++)
					{
						if ( (*lit)->getBox() == NULL && stricmp(name.c_str(), (*lit)->getName().c_str()) != NULL)
						{
							if ( stricmp((*lit)->getName().c_str(), snapTo.c_str()) == NULL )
							{
								result = isCloseTo( pos, size, (*lit)->actualx, (*lit)->actualy, (*lit)->actualwidth, (*lit)->actualheight, snapDistance);
							}
							else if ( stricmp((*lit)->labelGroup.c_str(), snapTo.c_str()) == NULL || stricmp("alllabels", snapTo.c_str()) == NULL )
							{
								result = isCloseTo( pos, size, (*lit)->actualx, (*lit)->actualy, (*lit)->actualwidth, (*lit)->actualheight, snapDistance);
							}
							if (result != 0)
								break;
						}
					}
				}
				#ifndef xLabelLight
				//----------------------------------------------------------------
					else if (labelType == 1)
					{	
						for( int i = 0; i != (GetSystemMetrics(SM_CXSCREEN)-2*DIx)/DIwidth; i++)
						{
							for (int j = 0; j != (GetSystemMetrics(SM_CYSCREEN)-2*DIy)/DIheight; j++)
							{
								result = isCloseTo( pos, size, DIx+i*DIwidth, DIy+j*DIheight, 0, 0, snapDistance);
						
								if (result != 0)
									break;
							}
							if (result != 0)
								break;
						}
					}
				//----------------------------------------------------------------
				#endif
			}

			if (result == 1)
			{
				SetCursorPos(pos.x+savedpt.x, newpt.y);
			}
			else if (result == 2)
			{
				SetCursorPos(newpt.x, pos.y+savedpt.y);
			}
			else if (result == 3)
			{
				SetCursorPos(pos.x+savedpt.x, pos.y+savedpt.y);
			}
		}

		POINT boxConverted;
		boxConverted.x = windowPos->x;
		boxConverted.y = windowPos->y;
		if (box != NULL)
			ScreenToClient(box, &boxConverted);

		PrefixedLSSetVariable(name, "CurrentX", IntToString( boxConverted.x ));
		PrefixedLSSetVariable(name, "CurrentY", IntToString( boxConverted.y ));
	}
}

void Label::onSize(int width, int height)
{
	actualheight = height;
	actualwidth = width;

	repaint(true);

	RedrawWindow(hWnd, NULL, NULL, RDW_UPDATENOW);

	if (labelType == 0)
	{
		PrefixedLSSetVariable(name, "CurrentWidth", IntToString( width ));
		PrefixedLSSetVariable(name, "CurrentHeight", IntToString( height ));
	}
}

void Label::relayMouseMessageToBox(UINT message, WPARAM wParam, LPARAM lParam)
{
	POINT pt;
	pt.x = GET_X_LPARAM(lParam);
	pt.y = GET_Y_LPARAM(lParam);

	if (box)
	{
		MapWindowPoints(hWnd, box, &pt, 1);
		PostMessage(box, message, wParam, MAKELPARAM((short) pt.x, (short) pt.y));
	}
	else
	{
		MapWindowPoints(hWnd, GetLitestepDesktop(), &pt, 1);
		PostMessage(GetLitestepDesktop(), message, wParam, MAKELPARAM((short) pt.x, (short) pt.y));
	}
}

void Label::onTimer(int timerID)
{
	switch(timerID)
	{
		case TIMER_MOUSETRACK:

			POINT pt;
			GetCursorPos(&pt);
			
			RECT rc;
			GetWindowRect(hWnd, &rc);

			if (PtInRect(&rc, pt))
			{
				ScreenToClient(hWnd, &pt);
				if (!labelHREFClickRegions.empty())
				{
					if (!checkEnterLeaveRegion(pt.x, pt.y, labelHREFClickRegions))
					{
						if (!originalTextBackup.empty())
						{
							current = currentBackup;
							setText(originalTextBackup);
						}
						SendMessage(hWnd, LM_SETCURSOR, 0, 0);
						mouseInsideHREF = false;
					}
				}
			}
			else if(!PtInRect(&rc, pt))
			{
				KillTimer(hWnd, TIMER_MOUSETRACK);
				mouseInside = false;
				mousePressed = false;
				if (!originalTextBackup.empty())
				{
					current = currentBackup;
					setText(originalTextBackup);
				}
				mouseInsideHREF = false;
				onMouseLeave();	
			}

		break;

		case TIMER_MOUSETRACK_MOVE:

			POINT newpt;
			GetCursorPos(&newpt);
			
			newpt.x = newpt.x-savedpt.x;
			newpt.y = newpt.y-savedpt.y;
			useHover = true;
			hoverActive = true;
		
			move(newpt.x, newpt.y, 0, 0, false, false);

			hasMoved = true;
		
		break;

		case TIMER_UPDATE:

			update();

		break;
		
		case TIMER_SCROLL:

			if (!frozen)
			{
				if (scroll == 1 || scroll == 2)
					scrollPosition -= scrollSpeed;
				else if (scroll == 3 || scroll == 4)
					scrollPosition += scrollSpeed;
				repaint();
			}

		break;

		case TIMER_AUTOHIDE:

			hide();

		break;

		case TIMER_ALPHAFADEIN:

			if (UpdateAlpha <= alphaTransparency-alphaFadeStep)
			{
				setAlpha(UpdateAlpha, false);
				UpdateAlpha += alphaFadeStep;
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEIN);
				if (UpdateAlpha != alphaTransparency)
					setAlpha(alphaTransparency, false);
			}
		
		break;

		case TIMER_ALPHAFADEOUT:

			if (UpdateAlpha >= alphaFadeStep)
			{
				setAlpha(UpdateAlpha, false);
				UpdateAlpha -= alphaFadeStep;
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEOUT);
				UpdateAlpha = 0;
				ShowWindow(hWnd, SW_HIDE);
			}

		break;

		case TIMER_FRAMEANIMATION:

			if (hoverActive && useHover)
			{
				if (frameCounter == hoverimageFrameCount)
				{
					frameCounter = 0;
					loopCounter++;
				}
				if (loopCounter == hoverimageLoop)
				{
					frameCounter = -1;
					loopCounter = 0;
					KillTimer(hWnd, TIMER_FRAMEANIMATION);
				}
				
			}
			else
			{
				if (frameCounter == imageFrameCount)
				{
					frameCounter = 0;
					loopCounter++;
				}
				if (loopCounter == imageLoop)
				{
					frameCounter = -1;
					loopCounter = 0;
					KillTimer(hWnd, TIMER_FRAMEANIMATION);
				}
			}
			frameCounter++;
			repaint(true);

		break;

		case TIMER_SOLIDPULSEANIMATION:

			repaint(true);
			
		break;
	}
}

bool Label::onWindowMessage(UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult)
{
	extern HWND messageHandler;
	
	if (moveable && moveKey != -1)
	{
		if (moveKey == 3)
			movemodifierkeyPressed = (0x8000 & GetKeyState(VK_CONTROL)) && (0x8000 & GetKeyState(VK_SHIFT));
		else if (moveKey == 2)
			movemodifierkeyPressed = (0x8000 & GetKeyState(VK_SHIFT));
		else if (moveKey == 1)
			movemodifierkeyPressed = (0x8000 & GetKeyState(VK_CONTROL));
		else 
			movemodifierkeyPressed = (0x8000 & GetKeyState(VK_CONTROL));
	}
	else if (moveable && moveKey == -1)
		movemodifierkeyPressed = true;

	if (moveable && !movemodifierkeyPressed)
		KillTimer(hWnd, TIMER_MOUSETRACK_MOVE);

	switch(message)
	{
		case LM_SETLABELTEXT:
		{
			setText(string((const char *) lParam), true);
			return true;
		}

		case LM_SETCURSOR:
		{
			if (!cursor && !cursorPath.empty())
			{
				if (strstr(cursorPath.c_str(), ":\\") != NULL)
					cursor = LoadCursorFromFile( cursorPath.c_str() );
				else
				{	
					char imagePath[MAX_PATH_LENGTH];
					VarExpansion( imagePath, "$lsimagefolder$" );
					strcat(imagePath, cursorPath.c_str());

					cursor = LoadCursorFromFile( imagePath );
				}

				if (!cursor)
					cursor = LoadCursor( 0, IDC_ARROW );

				SetCursor( cursor );
			}
			else if (!cursor && cursorPath.empty())
			{
				cursor = LoadCursor( 0, IDC_ARROW );
				SetCursor( cursor );
			}
			else if (lParam != NULL)
				SetCursor( LoadCursor(0, (const char *) lParam ) );
			else
				SetCursor( cursor );

			return true;
		}

		case WM_ACTIVATE:
		{
			if (labelType == 1)
			{
				if (wParam == WA_INACTIVE)
				{
					useHover = false;
					hoverActive = false;
					repaint(true);
				}
				else if (wParam == WA_CLICKACTIVE)
				{
					useHover = true;
					hoverActive = true;
					repaint(true);
					SendMessage(hWnd, WM_PAINT, 0, 0);
				}
			}
			
			return true;
		}

		case WM_COPYDATA:
		{
			COPYDATASTRUCT *cds = (COPYDATASTRUCT *) lParam;
			
			if(cds->dwData == LM_SETLABELTEXT)
				setText(string((const char *) cds->lpData));

			return false;
		}

		case WM_CLOSE:
		{
			lResult = 0;
			return true;
		}

		case WM_DESTROY:
		{
			if(box)		// FIXME: this is the old workaround
			{
				if (hWnd && TooltipHints)
				{
					removeHint(hWnd);
					DestroyWindow(TooltipHints);
					TooltipHints = 0;
				}

				if (hWnd)
				{
					KillTimer(hWnd, TIMER_MOUSETRACK_MOVE);
					KillTimer(hWnd, TIMER_MOUSETRACK);
					KillTimer(hWnd, TIMER_UPDATE);
					KillTimer(hWnd, TIMER_SCROLL);
					KillTimer(hWnd, TIMER_AUTOHIDE);
					KillTimer(hWnd, TIMER_ALPHAFADEIN);
					KillTimer(hWnd, TIMER_ALPHAFADEOUT);
					KillTimer(hWnd, TIMER_FRAMEANIMATION);
					KillTimer(hWnd, TIMER_SOLIDPULSEANIMATION);
				}

				box = NULL;
				hide();
			}
				
			hWnd = NULL;	// FIXME: dirty hack
			
			return false;
		}

		case WM_DROPFILES:
		{
			if(dropCommand.empty())	// we shouldn't get here... but somehow we did :p
				break;

			int intFileAction;

			char action[32], szTarget[MAX_PATH_LENGTH];
			char *buffers[] = {action, szTarget};

			LCTokenize(dropCommand.c_str(), buffers, 2, NULL);

			if (stricmp(action, "move") == 0)
				intFileAction = 1;
			else if (stricmp(action, "copy") == 0)
				intFileAction = 2;
			else if (stricmp(action, "delete") == 0)
				intFileAction = 0;
			else if (stricmp(action, "shortcut") == 0)
				intFileAction = 3;
			else
				intFileAction = -1;

			string check = szTarget;
			if (check.substr(1, check.length()-1) == "\\")
				check.append( "*" );
			else
				check.append( "\\*" );
				
			//Test Path for Existence
			WIN32_FIND_DATA FindData;
			HANDLE hFindHandle = FindFirstFile(check.c_str(), &FindData);

			if (intFileAction >= 0 && intFileAction < 3)
			{
				char *szSourceFiles = NULL;
				char buffer[MAX_PATH_LENGTH];
				int numFiles;
				UINT len,scursor = 0;

				numFiles = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, NULL);

				szSourceFiles = new char[MAX_PATH_LENGTH*numFiles+1];
				scursor = 0;

				for (int i=0;i<numFiles;i++)
				{
					len = DragQueryFile((HDROP) wParam, i, buffer, MAX_PATH_LENGTH);
					
					strcpy(szSourceFiles+scursor,buffer);
					scursor += len + 1;
				}
					
				DragFinish((HDROP) wParam);

				if( hFindHandle != INVALID_HANDLE_VALUE || intFileAction == 0 )
				{
					szSourceFiles[scursor] = 0;
					strcpy(buffer, szTarget);
					buffer[strlen(buffer)+1] = 0;
					
					SHFILEOPSTRUCT shfop;
					shfop.hwnd = NULL;
					if (intFileAction == 1)
						shfop.wFunc = FO_MOVE;
					else if (intFileAction == 2)
						shfop.wFunc = FO_COPY;
					else if (intFileAction == 0)
						shfop.wFunc = FO_DELETE;
					shfop.pFrom = szSourceFiles;
					shfop.pTo = buffer;
					shfop.fFlags = FOF_NOCONFIRMMKDIR | FOF_ALLOWUNDO;
					SHFileOperation(&shfop);
				}
				
				delete[] szSourceFiles;
			}
			else if (hFindHandle != INVALID_HANDLE_VALUE && intFileAction == 3)
			{
				CString path = szTarget;
				if (path.Right(1) != "\\")
					path += "\\";
				int numDropped;
				char szFile[MAX_PATH_LENGTH];
				
				numDropped = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, 0);
				for (int i = 0; i < numDropped; i++)
				{
					DragQueryFile((HDROP) wParam, i, (char *)&szFile, MAX_PATH_LENGTH);

					HRESULT hres; 
					IShellLink* psl; 

					hres = CoCreateInstance(CLSID_ShellLink, NULL,
					CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID *) &psl); 
					if (SUCCEEDED(hres))
					{
						IPersistFile* ppf; 
						psl->SetPath(szFile); 
						psl->SetDescription("");
 
						hres = psl->QueryInterface(IID_IPersistFile, (LPVOID*)&ppf); 
						if (SUCCEEDED(hres))
						{
							CString work = szFile;
							work = work.Right(work.GetLength()-work.ReverseFind('\\')-1);
							work = work.Left(work.Find("."));
							work += ".lnk";

							WCHAR wsz[MAX_PATH]; 
							MultiByteToWideChar(CP_ACP, 0, (path + work), -1, wsz, MAX_PATH_LENGTH);  
							hres = ppf->Save(wsz, TRUE); 
							ppf->Release(); 
						}
						psl->Release(); 
					}
				}
				DragFinish((HDROP) wParam);
			}
			else if (intFileAction == -1)
			{
				int numDropped, i;
				char szFile[MAX_PATH_LENGTH];
				
				numDropped = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, 0);
				for (i = 0; i < numDropped; i++)
				{
					DragQueryFile((HDROP) wParam, i, (char *)&szFile, MAX_PATH_LENGTH);
					if (szFile && szFile[0])
					{
						string fullCommand = dropCommand + " \"" + szFile + "\""; // FIXME: ugly hack
						LSExecute(hWnd, fullCommand.c_str(), SW_SHOWNORMAL);
					}
				}
				DragFinish((HDROP) wParam);
			}
			
			return 0;
		}
		
		case WM_LBUTTONDBLCLK:
		{
			if(leftDoubleClickCommand.empty())
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onLButtonDblClk((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_LBUTTONDOWN:
		{
			if(leftClickDownCommand.empty() && leftClickCommand.empty() && labelLeftClickRegions.empty() && labelHREFClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onLButtonDown((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_LBUTTONUP:
		{
			if(leftClickDownCommand.empty() && leftClickCommand.empty() && labelLeftClickRegions.empty() && labelHREFClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onLButtonUp((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_MBUTTONDBLCLK:
		{
			if(middleDoubleClickCommand.empty())
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onMButtonDblClk((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_MBUTTONDOWN:
		{
			if (focusOnEnter == 2)
				return true;

			if(middleClickDownCommand.empty() && middleClickCommand.empty() && labelMiddleClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onMButtonDown((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_MBUTTONUP:
		{
			if (focusOnEnter == 2)
			{
				focusOnEnter = 1;
				return true;
			}

			if(middleClickDownCommand.empty() && middleClickCommand.empty() && labelMiddleClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onMButtonUp((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_RBUTTONDBLCLK:
		{
			if(rightDoubleClickCommand.empty())
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onRButtonDblClk((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_RBUTTONDOWN:
		{
			if(rightClickDownCommand.empty() && rightClickCommand.empty() && labelRightClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onRButtonDown((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_RBUTTONUP:
		{
			if(rightClickDownCommand.empty() && rightClickCommand.empty() && labelRightClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onRButtonUp((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_MOUSEWHEEL:
		{
			if ((short)(HIWORD(wParam)) < 0)
			{
				if(wheelDownCommand.empty())
				{
					relayMouseMessageToBox(message, wParam, lParam);
					return true;
				}

				onWheelDown((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
				return true;
			}
			else
			{
				if(wheelUpCommand.empty())
				{
					relayMouseMessageToBox(message, wParam, lParam);
					return true;
				}
				
				onWheelUp((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
				return true;
			}
		}

		case WM_MOUSEMOVE:
		{
			onMouseMove((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		if ( !alphaMap || !IsOS(OS_2KXP) )
		{
			case WM_PAINT:
			{	
				PAINTSTRUCT ps;
				HDC hDC;

				if(!wParam)
					hDC = BeginPaint(hWnd, &ps);
				else
					hDC = (HDC) wParam;

				onPaint(hDC);

				if(!wParam)
					EndPaint(hWnd, &ps);
		
				return true;	
			}
		}

		case WM_ERASEBKGND:
		{
			return true;
		}

		case WM_SIZE:
		{
			onSize((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_TIMER:
		{
			onTimer((int) wParam);
			return true;
		}

		case WM_WINDOWPOSCHANGED:
		{
			onWindowPosChanged((WINDOWPOS *) lParam);
			return false;
		}
	}

	return false;
}

LRESULT Label::windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	Label *label = NULL;

	if(message == WM_NCCREATE)
	{
		label = (Label *) ((CREATESTRUCT *) lParam)->lpCreateParams;
		label->hWnd = hWnd;
		SetWindowLong(hWnd, 0, (LONG) label);
	}
	else
		label = (Label *) GetWindowLong(hWnd, 0);

	if(label)
	{
		LRESULT lResult = 0;

		if(label->onWindowMessage(message, wParam, lParam, lResult))
			return lResult;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

//------------------------------------------------------------
//Tooltips (Add, Update, Remove)
//------------------------------------------------------------

void Label::addHint(HWND hWnd, LPSTR caption)
{
	if (TooltipHints == 0)
		return;
	
	TOOLINFO ti;
	RECT clientRect;

	GetClientRect(hWnd, &clientRect);
	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = TTF_SUBCLASS;
	ti.hwnd = hWnd;
	ti.uId = 0;
	ti.rect = clientRect;
	ti.hinst = hInstance;
	ti.lpszText = caption;
	ti.lParam = 0;

	SendMessage(TooltipHints, TTM_ADDTOOL, 0, LPARAM(&ti));
}

void Label::updateHint(HWND hWnd, LPSTR caption, bool updatesize)
{
	if (TooltipHints == 0)
		return;	
	
	TOOLINFO ti;
	RECT clientRect;

	GetClientRect(hWnd, &clientRect);
	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = TTF_SUBCLASS;
	ti.hwnd = hWnd;
	ti.uId = 0;
	ti.rect = clientRect;
	ti.hinst = hInstance;
	ti.lpszText = caption;
	ti.lParam = 0;

	SendMessage(TooltipHints, TTM_UPDATETIPTEXT, 0, LPARAM(&ti));

	if (updatesize)
		SendMessage(TooltipHints, TTM_NEWTOOLRECT, 0, LPARAM(&ti));
}

void Label::removeHint(HWND hWnd)
{
	if (TooltipHints == 0)
		return;	
	
	TOOLINFO ti;
	RECT emptyRect = {0, 0, 0, 0};

	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = 0;
	ti.hwnd = hWnd;
	ti.uId = 0;
	ti.rect = emptyRect;
	ti.hinst = hInstance;
	ti.lpszText = NULL;
	ti.lParam = 0;

	SendMessage(TooltipHints, TTM_DELTOOL, 0, LPARAM(&ti));
}

//------------------------------------------------------------
//Initialize and Update AlphaTransparency
//------------------------------------------------------------

void Label::UpdateLabelAlpha(int newAlphaTransparency, bool updatePos)
{
	//Initialize LayeredWindow Function
	if (!alphaInitialized)
	{
		HMODULE hDLL = LoadLibrary ("user32");

		pUpdateLayeredWindow = 
					(PULW) GetProcAddress(hDLL, "UpdateLayeredWindow");

		pSetLayeredWindowAttributes = 
				(PSLWA) GetProcAddress(hDLL,"SetLayeredWindowAttributes");

		//Alpha Transparency Features Not Allowed
		if ( pUpdateLayeredWindow == NULL || pSetLayeredWindowAttributes == NULL )
		{
			alphaTransparency = 255;
			alphaMap = false;
			alphaFade = false;

			ModifyStyleEx(hWnd, WS_EX_LAYERED, 0 );

			layeredFlag = false;

			return;
		}

		alphaInitialized = true;
	}

	if (alphaMap)
	{
	//------------------------------------------------------------------
	
		bool alphaMap_Enabled = false;

		if (updatePos)
		{
			if(backgroundDC && backgroundBitmap)
			{
				backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
				DeleteObject(backgroundBitmap);
				backgroundBitmap = 0;
			}

			WINDOWPLACEMENT win_pos;
			GetWindowPlacement(hWnd, &win_pos);
			actualx = win_pos.rcNormalPosition.left;
			actualy = win_pos.rcNormalPosition.top;
			actualwidth = win_pos.rcNormalPosition.right - win_pos.rcNormalPosition.left;
			actualheight = win_pos.rcNormalPosition.bottom - win_pos.rcNormalPosition.top;
		}
			
		if(!backgroundBitmap)
		{
			if(!backgroundDC)
				backgroundDC = CreateCompatibleDC(0);

			BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=actualwidth;      // Set size you need
			alpha.bmiHeader.biHeight=actualheight;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

			backgroundBitmap = CreateDIBSection( backgroundDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

			SelectObject(backgroundDC, backgroundBitmap);
		}

		//BG Painting
		if (background)
		{
			if ( (!normalAnim && !hoverAnim) || (usePressed && pressedActive) || frameCounter == 0)
			{
				if (hoverActive && useHover)
					alphaMap_Enabled = background->apply(backgroundDC, 0, 0, actualwidth, actualheight, 1, 1);
				else if (pressedActive && usePressed)
					alphaMap_Enabled = background->apply(backgroundDC, 0, 0, actualwidth, actualheight, 1, 2);
				else
					alphaMap_Enabled = background->apply(backgroundDC, 0, 0, actualwidth, actualheight, 1, 0);
			}
			else
			{
				if (hoverActive && useHover)
					alphaMap_Enabled = background->applyAnim(backgroundDC, 0, 0, actualwidth, actualheight, 1, 1, frameCounter);
				else if (pressedActive && usePressed)
					alphaMap_Enabled = background->applyAnim(backgroundDC, 0, 0, actualwidth, actualheight, 1, 1, frameCounter);
				else
					alphaMap_Enabled = background->applyAnim(backgroundDC, 0, 0, actualwidth, actualheight, 1, 0, frameCounter);
			}
		}

		//Text Painting
		if (!text.empty() && labelType == 0)
			onTextPaint(backgroundDC, actualwidth, actualheight);

		//Prepare Combined Image
		if (alphaMap_Enabled && labelType == 0)
		{
			BITMAP bm;
			GetObject(backgroundBitmap, sizeof(BITMAP), &bm);

			BYTE *pPixel = (LPBYTE)bm.bmBits;

			for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
			{
				//Convert Text to Full Opacity
				if (pPixel[3] == 0)
					pPixel[3] = 255;
				//Remove Prepared Alpha (1 back to 0)
				if (pPixel[3] == 1)
					pPixel[3] = 0;

				pPixel+= 4;
			}
		}

		POINT ptWindowScreenPosition = {actualx, actualy};
		SIZE szWindow = {actualwidth, actualheight};

		POINT ptSrc = {0,0};

		BLENDFUNCTION blendPixelFunction = { AC_SRC_OVER, 0, newAlphaTransparency, (alphaMap_Enabled ? AC_SRC_ALPHA : 0) };

		pUpdateLayeredWindow(hWnd, NULL, &ptWindowScreenPosition, &szWindow, backgroundDC, &ptSrc, 0, &blendPixelFunction, (alphaMap_Enabled ? ULW_ALPHA : ULW_OPAQUE) );
	//------------------------------------------------------------------
	}
	else
		pSetLayeredWindowAttributes(hWnd, 0, newAlphaTransparency, LWA_ALPHA);
}

//------------------------------------------------------------
//Reorder "ShortcutLabels" to keep compatible to sc2
//------------------------------------------------------------
#ifndef xLabelLight
//----------------------------------------------------------------
	void Label::ReorderZOrder()
	{
		for(LabelListIterator iter = shortcutlabelList.begin(); iter != shortcutlabelList.end(); iter++)
		{
			if((*iter)->getName() == this->getName())
			{
				iter++;
				if (iter != NULL)
				{
					SetWindowPos(hWnd, (*iter)->hWnd,
						0, 0, 0, 0,
						SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
				}
				break;
			}
		}
	}
//----------------------------------------------------------------
#endif

int Label::isCloseTo(POINT &pos, const POINT size, const int x, const int y, const int cx, const int cy, const int span)
{
	bool internalX = false;
	bool internalY = false;

	if ((pos.x > x-span) && (pos.x < x+span))
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) && !DISnapTo)
		{
			pos.x = x;
			internalX = true;
		}
	}
	else if ((pos.x > x+cx-span ) && (pos.x < x+cx+span) && !DISnapTo)
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x+cx;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x+cx;
			internalX = true;
		}
	}
	else if ((pos.x+size.x > x-span ) && (pos.x+size.x < x+span) && !DISnapTo)
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x-size.x;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x-size.x;
			internalX = true;
		}
	}
	else if ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span) && !DISnapTo)
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x+cx-size.x;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x+cx-size.x;
			internalX = true;
		}
	}

	if ((pos.y > y-span) && (pos.y < y+span))
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) && !DISnapTo)
		{
			pos.y = y;
			internalY = true;
		}
	}
	else if ((pos.y > y+cy-span ) && (pos.y < y+cy+span) && !DISnapTo)
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y+cy;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y+cy;
			internalY = true;
		}
	}
	else if ((pos.y+size.y > y-span ) && (pos.y+size.y < y+span) && !DISnapTo)
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y-size.y;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y-size.y;
			internalY = true;
		}
	}
	else if ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span) && !DISnapTo)
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y+cy-size.y;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y+cy-size.y;
			internalY = true;
		}
	}

	if (!internalX && !internalY)
		return 0;
	else if (internalX && !internalY)
		return 1;
	else if (!internalX && internalY)
		return 2;
	else
		return 3;
}

void Label::ontemplateButtonDown(int actualMouseClick, const string &clickDownCommand)
{
	//Stop Animation Timer (Normal & Hover)
	KillTimer(hWnd, TIMER_FRAMEANIMATION);

	if(moveable && moveButton == actualMouseClick)
	{
		if(!mousePressed && !movemodifierkeyPressed)
			mousePressed = true;
		else if (!mousePressed && movemodifierkeyPressed)
		{
			if (box)
				return;
			mousePressed = true;
			moveButtonPressed = actualMouseClick;
			GetCursorPos(&savedpt);
			ScreenToClient(hWnd, &savedpt);
			SetCapture(hWnd);
			SendMessage(hWnd, LM_SETCURSOR, 0, (long) IDC_SIZEALL);
		}
	}
	else
	{
		if(!mousePressed)
			mousePressed = true;
	}

	if( !clickDownCommand.empty() )
		xLSExecute(hWnd, clickDownCommand.c_str(), SW_SHOWNORMAL);

	if (usePressed)
	{
		hoverActive = false;
		pressedActive = true;
		repaint(true);
	
		if (transparencyMode == 1 && labelType != 1)
			updateRegion();
	}
	#ifndef xLabelLight
	//----------------------------------------------------------------
		if (labelType == 2)
			ReorderZOrder();
	#endif
	//----------------------------------------------------------------
}

void Label::ontemplateButtonUp(const int &x, const int &y, int actualMouseClick, const StringList &labelClickRegions, const string &clickCommand)
{
	if (labelType == 2 && !pressedSound.empty())
		ParseBangCommand(hWnd, "!playsound", pressedSound.c_str());

	if(moveable && moveButton == actualMouseClick)
	{
		if(mousePressed && !movemodifierkeyPressed)
		{
			SendMessage(hWnd, LM_SETCURSOR, 0, 0);
			//To be on the save side
			ReleaseCapture();
			KillTimer(hWnd, TIMER_MOUSETRACK_MOVE);
			
			if (!checkRegion(x, y, labelClickRegions) && !checkRegion(x, y, labelHREFClickRegions))
			{
				if( !clickCommand.empty() )
					xLSExecute(hWnd, clickCommand.c_str(), SW_SHOWNORMAL);
			}

			#ifndef xLabelLight
			//----------------------------------------------------------------
				if ( rememberPosition )
				{
					string bang;
					bang = ("!labeldesktopsave " + iconCommand);
					LSExecute(hWnd, bang.c_str(), SW_SHOWNORMAL);
				}
			//----------------------------------------------------------------
			#endif
			
			mousePressed = false;
			moveButtonPressed = 0;
		}
		else if (mousePressed && movemodifierkeyPressed)
		{
			SendMessage(hWnd, LM_SETCURSOR, 0, 0);
			//To be on the save side
			ReleaseCapture();
			KillTimer(hWnd, TIMER_MOUSETRACK_MOVE);

			if (!checkRegion(x, y, labelClickRegions) && !checkRegion(x, y, labelHREFClickRegions))
			{
				if( !clickCommand.empty() )
					xLSExecute(hWnd, clickCommand.c_str(), SW_SHOWNORMAL);
			}

			#ifndef xLabelLight
			//----------------------------------------------------------------
				if ( rememberPosition )
				{
					string bang;
					bang = ("!labeldesktopsave " + iconCommand);
					LSExecute(hWnd, bang.c_str(), SW_SHOWNORMAL);
				}
			//----------------------------------------------------------------
			#endif

			mousePressed = false;
			moveButtonPressed = 0;
		}

		if(hasMoved && labelType == 0 && !moveCommand.empty())
			xLSExecute(hWnd, moveCommand.c_str(), SW_SHOWNORMAL);
		hasMoved = false;
	}
	else 
	{
		//To be on the save side
		ReleaseCapture();
		KillTimer(hWnd, TIMER_MOUSETRACK_MOVE);

		if(mousePressed && !checkRegion(x, y, labelClickRegions) && !checkRegion(x, y, labelHREFClickRegions))
		{
			if(!clickCommand.empty() && stricmp(clickCommand.c_str(), ".context") != NULL && stricmp(clickCommand.c_str(), ".properties") != NULL)
				xLSExecute(hWnd, clickCommand.c_str(), SW_SHOWNORMAL);
			#ifndef xLabelLight
			//----------------------------------------------------------------
				else if (!clickCommand.empty() && stricmp(clickCommand.c_str(), ".context") == NULL)
					ContextMenu(iconCommand.c_str());
				else if (!clickCommand.empty() && stricmp(clickCommand.c_str(), ".properties") == NULL)
				{
					SHELLEXECUTEINFO sei = {sizeof(sei)};
					sei.fMask = SEE_MASK_INVOKEIDLIST;
					sei.lpVerb = "properties";
					sei.lpFile = iconCommand.c_str();
					ShellExecuteEx(&sei);
				}
			//----------------------------------------------------------------
			#endif
		}
		mousePressed = false;
		moveButtonPressed = 0;
	}

	if (usePressed)
	{
		hoverActive = true;
		pressedActive = false;
		repaint(true);

		if (transparencyMode == 1 && labelType != 1)
			updateRegion();
	}

	//Animation RESTART
	if (useHover)
	{
		//Restart Hover Anim
		if (hoverAnim)
		{
			frameCounter = 1;
			loopCounter = 0;
			SetTimer(hWnd, TIMER_FRAMEANIMATION, hoverimageFrameDelay, NULL);
		}
	}
	//Restart Normal Anim
	else if (normalAnim)
	{
		frameCounter = 1;
		loopCounter = 0;
		SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
	}

	#ifndef xLabelLight
	//----------------------------------------------------------------
		if (labelType == 2)
			ReorderZOrder();
	//----------------------------------------------------------------
	#endif
}

//MessageBox(NULL, "Couldn't initialise PluginFunctions!\nMaybe you use an older xanim.dll version, than xlabel.dll version!", "xLabel Plugin Informations", MB_SETFOREGROUND);
