#if !defined(__LABEL_H)
#define __LABEL_H

#define LM_SETLABELTEXT 9600
#define LM_SETCURSOR    9610

#define GET_X_LPARAM(lp) ((int)(short)LOWORD(lp))
#define GET_Y_LPARAM(lp) ((int)(short)HIWORD(lp))

#ifndef WM_MOUSEWHEEL
	#define WM_MOUSEWHEEL 0x020A
#endif

//Ballon Tooltip
#ifndef TTS_BALLOON
#define TTS_BALLOON 0x40
#endif

//Alpha Transparency
#ifndef WS_EX_LAYERED
#define WS_EX_LAYERED           0x00080000L
#define LWA_COLORKEY            0x00000001L
#define ULW_COLORKEY            0x00000001L
#define LWA_ALPHA               0x00000002L
#define ULW_ALPHA               0x00000002L
#define ULW_OPAQUE              0x00000003L
#endif // WS_EX_LAYERED

#ifndef AC_SRC_ALPHA
#define AC_SRC_ALPHA            0x00000001L
#endif

#define ALIGN_LEFT 0
#define ALIGN_TOP 0
#define ALIGN_CENTER 1
#define ALIGN_RIGHT 2
#define ALIGN_BOTTOM 2

class xFont;
class Texture;

//Alpha Transparency
typedef BOOL (WINAPI *PULW)(HWND hwnd, HDC hdcDst, POINT *pptDst, SIZE *psize, HDC hdcSrc, POINT *pptSrc, COLORREF crKey, BLENDFUNCTION *pblend, DWORD dwFlags);
typedef DWORD (WINAPI *PSLWA)(HWND, DWORD, BYTE, DWORD);

static PULW pUpdateLayeredWindow = NULL;
static PSLWA pSetLayeredWindowAttributes = NULL;

static BOOL alphaInitialized = FALSE;
//-----------------------------------------------------

class Label
{
public:

	Label(const string &name);
	virtual ~Label();

	void load(HINSTANCE hInstance, HWND box = 0, bool dropShadow = false);
	void reconfigure(bool refresh = false);

	#ifndef xLabelLight
	//----------------------------------------------------------------
		void desktopload(HINSTANCE hInstance, string configLine = "");
		void shortcutload(HINSTANCE hInstance, string configLine = "", int lastx = 0, int lasty = 0, HWND box = 0);

		void desktopreconfigure(string configLine);
		//Return TRUE, if ToolTips are used
		bool shortcutreconfigure(string configLine, int lastx, int lasty);

		//Desktop(/Shortcut)
		string iconCommand;
		bool rememberPosition;
		bool easyDesktopFocus;
	//----------------------------------------------------------------
	#endif

	Texture *getBackground() const { return background; }
	xFont *getFont() const { return font; }

	bool isAlwaysOnTop() const { return alwaysOnTop; }
	bool isGhosted() const { return ghosted; }
	bool isVisible() const { return visible; }
	bool isStartHidden() const { return startHidden; }
	bool isLayeredFlagSet() const { return layeredFlag; }

	int getX() const { return actualx; }
	int getY() const { return actualy; }
	int getHeight() const { return actualheight; }
	int getWidth() const { return actualwidth; }

	int getTransparencyMode() const { return transparencyMode; }

	int getScrolling() const { return scroll; }

	const HWND &getBox() const { return box; }
	const HWND &gethWnd() const { return hWnd; }
	const string &getName() const { return name; }
	const string &getText() const { return text; }

	void setBox(HWND newparent);
	void setBackground(Texture *newbackground);
	void setFont(xFont *newfont);
	void setText(const string &text, bool updateBackup = false);

	void setAlwaysOnTop(bool alwaysOnTop);
	void setGhosted(bool ghosted);

	void setScrolling(int scrolling);
	void setScrollLimit(int limit);
	
	void setAlpha(int newAlphaTransparency = 255, bool updateValue = true);

	void setAnimation(bool start, int loops = -1, int startFrame = 1);

	bool checkEnterLeaveRegion(int x, int y, StringList inputlist);
	bool checkRegion(int x, int y, StringList inputlist);
	
	void repaint(bool invalidateCache = false);
	void updateRegion();

	void move(int newx, int newy, int steps = 0, int time = 0, bool moveBy = false, bool command = true);
	void resize(int newwidth, int newheight, int steps = 0, int time = 0, bool resizeBy = false);
	void reposition(int newx, int newy, int newwidth, int newheight, int steps = 0, int time = 0, bool repositionBy = false);

	void hide(bool execute = false);
	void show(bool execute = false);
	void showHide(double timeout);

	void previous();
	void next();
	void update();

	void clipboardCopy(string prefix);
	void clipboardPaste(string prefix);

	bool bUseFahrenheit;

	string resizeCommand;

	string configSavePath;

	//Scroll AddOns
	bool frozen;
	int scrollBackup;

	//Label/Shortcut Group
	string labelGroup;

	//AutoSize
	int autoWidthMode;
	int autoHeightMode;

	//ModuleHook
	StringList moduleHook;

	bool DISnapTo;
	#ifndef xLabelLight
	//----------------------------------------------------------------
		int DIx;
		int DIy;
		int DIwidth;
		int DIheight;
	//----------------------------------------------------------------
	#endif

private:

	HCURSOR cursor;
	HCURSOR handcursor;
	string cursorPath;

	HWND hWnd;
	HINSTANCE hInstance;
	HWND box;

	HDC backgroundDC;
	HDC bufferDC;
	HBITMAP backgroundBitmap;
	HBITMAP bufferBitmap;

	bool startHidden;

	bool hoverActive;
	bool pressedActive;

	bool useHover;
	bool usePressed;

	bool normalAnim;
	bool hoverAnim;

	int lastImageMode;
	int overblendStyle;

	int borderMethod;
	int hoverborderMethod;
	int pressedborderMethod;

	string hoverSound;
	string pressedSound;

	Texture *background;
	xFont *font;

	bool alwaysOnTop;
	bool visible;
	bool bUsingDefSkin;

	int actualheight;
	int actualwidth;
	int actualx;
	int actualy;

	int scrollPadLength;
	int scrollInterval;
	int scrollSpeed;
	int scroll;
	int scrollPosition;
	UINT scrollLimit;

	//AutoSize
	int autoMinWidth;
	int autoMinHeight;
	int autoMaxWidth;
	int autoMaxHeight;
	int startCenterX;
	int startCenterY;

	int transparencyMode;
	bool transparencyNeeded;

	bool layeredFlag;
	int alphaTransparency;
	bool alphaMap;
	int alphaFade;
	bool ghosted;

	int updateInterval;
	bool alwaysUpdateContent;

	int align;
	int vertAlign;
	int leftBorder;
	int topBorder;
	int rightBorder;
	int bottomBorder;

	string name;

	bool dynamicText;
	vector<string> originalText;
	string originalTextBackup;
	int current;
	int currentBackup;
	bool noTextSplit;
	string text;
	vector<string> hrefconfig;
	bool noEscape;

	bool dynamicTooltip;
	string originalTooltip;
	char tooltip[MAX_PATH_LENGTH];
	int tooltipmaxwidth;
	bool balloontooltip;

	int labelType;

	int UpdateAlpha;

	bool mousePressed;
	bool mouseInside;

	bool mouseInsideHREF;

	int focusOnEnter;

	string leftClickCommand;
	string leftClickDownCommand;
	string leftDoubleClickCommand;
	string middleClickCommand;
	string middleClickDownCommand;
	string middleDoubleClickCommand;
	string rightClickCommand;
	string rightClickDownCommand;
	string rightDoubleClickCommand;
	string wheelDownCommand;
	string wheelUpCommand;
	string enterCommand;
	string leaveCommand;
	string dropCommand;
	string textChangeCommand;
	string moveCommand;

	string hideCommand;
	string showCommand;

	POINT savedpt;

	string snapTo;
	int snapDistance;

	bool hasMoved;
	bool movemodifierkeyPressed;
	int moveButtonPressed;

	bool moveable;
	int moveKey;
	int moveButton;

	StringList labelLeftClickRegions;
	StringList labelRightClickRegions;
	StringList labelMiddleClickRegions;

	StringList labelHREFClickRegions;

	//Animations
	int frameCounter;
	int loopCounter;

	int imageLoop;
	int imageFrameDelay;
	int imageFrameCount;
	int hoverimageLoop;
	int hoverimageFrameDelay;
	int hoverimageFrameCount;

	int alphaFadeStep;
	int alphaFadeDelay;

	int solidPulseDelay;

protected:

	void ontemplateButtonDown(int actualMouseClick, const string &clickDownCommand);
	void ontemplateButtonUp(const int &x, const int &y, int actualMouseClick, const StringList &labelClickRegions, const string &clickCommand);

	void autoSize();

	#ifndef xLabelLight
	//----------------------------------------------------------------
		void ReorderZOrder();
	//----------------------------------------------------------------
	#endif
	
	int isCloseTo(POINT &pos, const POINT size, int x, int y, const int cx, const int cy, const int span);

	HWND TooltipHints;

	void addHint(HWND hWnd, LPSTR caption);
	void updateHint(HWND hWnd, LPSTR caption, bool updatesize = false);
	void removeHint(HWND hWnd);

	void UpdateLabelAlpha(int newAlphaTransparency, bool updatePos);

	void onTextPaint(HDC desthdc, int width, int height);

	void relayMouseMessageToBox(UINT message, WPARAM wParam, LPARAM lParam);

	virtual void onLButtonDblClk(int x, int y);
	virtual void onLButtonDown(int x, int y);
	virtual void onLButtonUp(int x, int y);
	virtual void onMButtonDblClk(int x, int y);
	virtual void onMButtonDown(int x, int y);
	virtual void onMButtonUp(int x, int y);
	virtual void onRButtonDblClk(int x, int y);
	virtual void onRButtonDown(int x, int y);
	virtual void onRButtonUp(int x, int y);
	virtual void onWheelDown(int x, int y);
	virtual void onWheelUp(int x, int y);
	virtual void onMouseEnter();
	virtual void onMouseLeave();
	virtual void onMouseMove(int x, int y);
	virtual void onPaint(HDC hDC);
	virtual void onSize(int width, int height);
	virtual void onTimer(int timerID);
	virtual void onWindowPosChanged(WINDOWPOS *windowPos);

	virtual bool onWindowMessage(UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult);

public:

	static LRESULT WINAPI windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
};

#endif
