#include "StdAfx.h"
#include "ShortcutTexture.h"

ShortcutTexture::ShortcutTexture()
{
	memoryDC = 0;
	memoryBitmap = 0;

	memoryHoverDC = 0;
	memoryHoverBitmap = 0;

	memoryPressedDC = 0;
	memoryPressedBitmap = 0;

	transparencyNeeded = false;

	has_AlphaMap = false;
	has_hoverAlphaMap = false;
	has_pressedAlphaMap = false;
}

ShortcutTexture::~ShortcutTexture()	
{
	if(memoryBitmap)
	{
		memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
		DeleteObject(memoryBitmap);
	}
	if(memoryDC)
		DeleteDC(memoryDC);

	if(memoryHoverBitmap)
	{
		memoryHoverBitmap = (HBITMAP) SelectObject(memoryHoverDC, memoryHoverBitmap);
		DeleteObject(memoryHoverBitmap);
	}
	if(memoryHoverDC)
		DeleteDC(memoryHoverDC);

	if(memoryPressedBitmap)
	{
		memoryPressedBitmap = (HBITMAP) SelectObject(memoryPressedDC, memoryPressedBitmap);
		DeleteObject(memoryPressedBitmap);
	}
	if(memoryPressedDC)
		DeleteDC(memoryPressedDC);
}

void ShortcutTexture::configure(const string &images, const string &useAlphaMap)
{
	char normal[64], hover[64], pressed[64];
	char *buffers[] = {normal, hover, pressed};

	LCTokenize(images.c_str(), buffers, 3, NULL);

	memoryBitmap = LoadLSImage(normal, 0);

	if ( stricmp(useAlphaMap.c_str(), "true") == NULL )
	{
		BITMAP bm;
		GetObject(memoryBitmap, sizeof(BITMAP), &bm);
		has_AlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

		if (has_AlphaMap)
			PrepareAlphaChannel(memoryBitmap);
	}

	GetLSBitmapSize(memoryBitmap, &imageWidth, &imageHeight);

	memoryDC = CreateCompatibleDC(0);
	SelectObject(memoryDC, memoryBitmap);

	if ( stricmp(hover, ".none") != 0 )
	{
		memoryHoverBitmap = LoadLSImage(hover, 0);

		if ( stricmp(useAlphaMap.c_str(), "true") == NULL )
		{
			BITMAP bm;
			GetObject(memoryHoverBitmap, sizeof(BITMAP), &bm);
			has_hoverAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

			if (has_hoverAlphaMap)
				PrepareAlphaChannel(memoryHoverBitmap);
		}

		GetLSBitmapSize(memoryHoverBitmap, &hoverimageWidth, &hoverimageHeight);

		memoryHoverDC = CreateCompatibleDC(0);
		SelectObject(memoryHoverDC, memoryHoverBitmap);
	}

	if ( stricmp(pressed, ".none") != 0 )
	{
		memoryPressedBitmap = LoadLSImage(pressed, 0);

		if ( stricmp(useAlphaMap.c_str(), "true") == NULL )
		{
			BITMAP bm;
			GetObject(memoryPressedBitmap, sizeof(BITMAP), &bm);
			has_pressedAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

			if (has_pressedAlphaMap)
				PrepareAlphaChannel(memoryPressedBitmap);
		}

		GetLSBitmapSize(memoryPressedBitmap, &pressedimageWidth, &pressedimageHeight);

		memoryPressedDC = CreateCompatibleDC(0);
		SelectObject(memoryPressedDC, memoryPressedBitmap);
	}

	if (!has_AlphaMap && stricmp(useAlphaMap.c_str(), "false") == NULL)
		checkMagicPink(memoryDC, imageWidth, imageHeight);
	if (memoryHoverDC && !has_hoverAlphaMap && stricmp(useAlphaMap.c_str(), "false") == NULL)
		checkMagicPink(memoryHoverDC, hoverimageWidth, hoverimageHeight);
	if (memoryPressedDC && !has_pressedAlphaMap && stricmp(useAlphaMap.c_str(), "false") == NULL)
		checkMagicPink(memoryPressedDC, pressedimageWidth, pressedimageHeight);
}

bool ShortcutTexture::apply(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType)
{
	if (imageType == 0 && memoryBitmap == 0)
		return false;
	else if ( imageType == 1 && memoryHoverBitmap == 0 )
		imageType = 0;
	else if ( imageType == 2 && memoryPressedBitmap == 0 && memoryHoverBitmap != 0)
		imageType = 1;
	else if ( imageType == 2 && memoryPressedBitmap == 0 && memoryHoverBitmap == 0)
		imageType = 0;

	SetStretchBltMode(hDC, STRETCH_DELETESCANS);

	if (imageType == 0)
	{
		MultiBlt(hDC, 0, 0, width, height,
			memoryDC, 0, 0, imageWidth, imageHeight,
			0, 0, 0, 0,
			MULTIBLT_STRETCH, SRCCOPY);
	}
	else if (imageType == 1)
	{
		MultiBlt(hDC, 0, 0, width, height,
			memoryHoverDC, 0, 0, hoverimageWidth, hoverimageHeight,
			0, 0, 0, 0,
			MULTIBLT_STRETCH, SRCCOPY);
	}
	else if (imageType == 2)
	{
		MultiBlt(hDC, 0, 0, width, height,
			memoryPressedDC, 0, 0, pressedimageWidth, pressedimageHeight,
			0, 0, 0, 0,
			MULTIBLT_STRETCH, SRCCOPY);
	}

	if (imageType == 0)
		return has_AlphaMap;
	else if ( imageType == 1 && memoryHoverBitmap != 0 )
		return has_hoverAlphaMap;
	else if (imageType == 2 && memoryPressedBitmap != 0)
		return has_pressedAlphaMap;
	else 
		return false;
}

void ShortcutTexture::PrepareAlphaChannel(HBITMAP &bmp)
{
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	for (int i = 0; i < bm.bmHeight * bm.bmWidth; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//PreMultiply
		pPixel[0] = pPixel[0] * pPixel[3] / 255;
		pPixel[1] = pPixel[1] * pPixel[3] / 255;
		pPixel[2] = pPixel[2] * pPixel[3] / 255;

		pPixel+= 4;
	}
}

void ShortcutTexture::checkMagicPink(HDC destHDC, int width, int height)
{
	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;
	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);

			if( ( r == 255 && g == 0 && b == 255 ) )
			{
				transparencyNeeded = true;
				return;
			}
		}
	}
}