#include "StdAfx.h"
#include "xFont.h"
#include "HTMLDrawer.h"
#include "HTMLFont.h"

xFont::xFont()
{
}

xFont::~xFont()
{
}

void xFont::configure(const string &prefix, xFont *baseFont)
{
	shadow = GetxRCBool(prefix, "FontShadow", baseFont ? baseFont->shadow : false);
	shadowColor = GetxRCColor(prefix, "FontShadowColor", baseFont ? baseFont->shadowColor : RGB(128, 128, 128));
	shadowX = GetxRCInt(prefix, "FontShadowX", baseFont ? baseFont->shadowX : 1);
	shadowY = GetxRCInt(prefix, "FontShadowY", baseFont ? baseFont->shadowY : 1);

	textoutline = GetxRCBool(prefix, "FontOutline", baseFont ? baseFont->textoutline : false);
	textoutlineColor = GetxRCColor(prefix, "FontOutlineColor", baseFont ? baseFont->textoutlineColor : RGB(128, 128, 128));

	name = GetxRCString(prefix, "Font", baseFont ? baseFont->name : "Arial");
	height = GetxRCInt(prefix, "FontHeight", baseFont ? baseFont->height : 15, 0);
	color = GetxRCColor(prefix, "FontColor", baseFont ? baseFont->color : RGB(0, 0, 0));

	bold = GetxRCBool(prefix, "FontBold", baseFont ? baseFont->bold : false);
	italic = GetxRCBool(prefix, "FontItalic", baseFont ? baseFont->italic : false);
	underline = GetxRCBool(prefix, "FontUnderline", baseFont ? baseFont->underline : false);

	smooth = GetxRCBool(prefix, "FontSmoothing", baseFont ? baseFont->smooth : true);
	lineBreak = GetxRCBool(prefix, "AutoLineBreak", baseFont ? baseFont->lineBreak : false);

	newlineSpace = GetxRCInt(prefix, "NewLineSpace", baseFont ? baseFont->newlineSpace : 0);
}

void xFont::apply(HDC hDC, int left, int right, int top, int bottom, const string &text, int align, int vertAlign, StringList &labelHREFClickRegions, bool desktopLabel)
{
	//Convert into MFC variables
	CDC tmpCDC;
	tmpCDC.Attach(hDC);

	bool needReallyHtml = true;
	CString strText = text.c_str();
	strText.Replace("<br><br>", "<br> <br>");

	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, name.c_str());
	lf.lfHeight = height;
    lf.lfCharSet = DEFAULT_CHARSET;

	CHTMLFont fntDefault;
	fntDefault.SetBold(bold);
	fntDefault.SetItalic(italic);
	fntDefault.SetUnderline(underline);
	if (desktopLabel)
		fntDefault.SetSmooth(false);
	else
		fntDefault.SetSmooth(smooth);
	fntDefault.SetName(lf.lfFaceName);
	fntDefault.SetCharSet(lf.lfCharSet);
	fntDefault.SetSize(lf.lfHeight);

	short nAlignment = align;
	short nVAlignment = vertAlign;

	CRect rctPosition; 

	// Call to CHTMLDrawer
	if (shadow && !textoutline)
	{
		rctPosition.left = left+shadowX;
		rctPosition.right = right;
		rctPosition.top = top+shadowY;
		rctPosition.bottom = bottom;

		fntDefault.SetColor(shadowColor);

		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								rctPosition,
								nAlignment,
								nVAlignment,
								(desktopLabel ? true : lineBreak),
								newlineSpace,
								labelHREFClickRegions, needReallyHtml);
	}

	if (textoutline)
	{
		rctPosition.left = left;
		rctPosition.right = right;
		rctPosition.top = top;
		rctPosition.bottom = bottom;

		fntDefault.SetColor(textoutlineColor);

		CRect r;
		CopyRect(&r, &rctPosition);
		r.OffsetRect(1, 1);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								(desktopLabel ? true : lineBreak),
								newlineSpace,
								labelHREFClickRegions, needReallyHtml);
		r.OffsetRect(-1, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								(desktopLabel ? true : lineBreak),
								newlineSpace,
								labelHREFClickRegions, needReallyHtml);
		r.OffsetRect(-1, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								(desktopLabel ? true : lineBreak),
								newlineSpace,
								labelHREFClickRegions, needReallyHtml);
		r.OffsetRect(0, -1);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								(desktopLabel ? true : lineBreak),
								newlineSpace,
								labelHREFClickRegions, needReallyHtml);
		r.OffsetRect(0, -1);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								(desktopLabel ? true : lineBreak),
								newlineSpace,
								labelHREFClickRegions, needReallyHtml);
		r.OffsetRect(1, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								(desktopLabel ? true : lineBreak),
								newlineSpace,
								labelHREFClickRegions, needReallyHtml);
		r.OffsetRect(1, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								(desktopLabel ? true : lineBreak),
								newlineSpace,
								labelHREFClickRegions, needReallyHtml);
		r.OffsetRect(0, 1);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								(desktopLabel ? true : lineBreak),
								newlineSpace,
								labelHREFClickRegions, needReallyHtml);
	}

	rctPosition.left = left;
	rctPosition.right = right;
	rctPosition.top = top;
	rctPosition.bottom = bottom;

	fntDefault.SetColor(color);

	CHTMLDrawer::DrawText(&tmpCDC,
							strText,
							fntDefault,
							rctPosition,
							nAlignment,
							nVAlignment,
							(desktopLabel ? true : lineBreak),
							newlineSpace,
							labelHREFClickRegions, needReallyHtml);

	tmpCDC.Detach();
}

POINT xFont::measure(HDC hDC, int left, int right, const string &text, int align, int vertAlign)
{
	//Convert into MFC variables
	CDC tmpCDC;
	tmpCDC.Attach(hDC);

	CString strText = text.c_str();
	strText.Replace("<br><br>", "<br> <br>");

	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, name.c_str());
	lf.lfHeight = height;
    lf.lfCharSet = DEFAULT_CHARSET;

	CHTMLFont fntDefault;
	fntDefault.SetBold(bold);
	fntDefault.SetItalic(italic);
	fntDefault.SetUnderline(underline);
	fntDefault.SetSmooth(smooth);
	fntDefault.SetName(lf.lfFaceName);
	fntDefault.SetCharSet(lf.lfCharSet);
	fntDefault.SetSize(lf.lfHeight);

	short nAlignment = align;
	short nVAlignment = vertAlign;

	CRect rctPosition; 
	if (lineBreak)
	{
		rctPosition.left = left;
		rctPosition.right = right;
	}
	else
	{
		rctPosition.left = 0;
		rctPosition.right = 0;
	}
	rctPosition.top = 0;
	rctPosition.bottom = 0;

	// Call to CHTMLDrawer
	POINT returnValues = CHTMLDrawer::MeasureText(&tmpCDC,
													strText,
													fntDefault,
													rctPosition,
													nAlignment,
													nVAlignment,
													lineBreak,
													newlineSpace);

	tmpCDC.Detach();
	
	return returnValues;
}

void xFont::setColor(int aColor, bool shadow)
{
	if (shadow)
		shadowColor = aColor;
	else
		color = aColor;
}

void xFont::setShadow(bool shadowEnabled)
{
	shadow = shadowEnabled;
}

